# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

from ..baseWidget import *
from ..logics import *

from wallaby.pf.peer.imageViewer import *
from wallaby.pf.peer.imageEditor import *

class ImageLabel(QtGui.QLabel, BaseWidget, EnableLogic, ViewLogic, EditLogic):
    __metaclass__ = QWallabyMeta

    def __init__(self, *args):
        QtGui.QLabel.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QLabel, *args)
        EnableLogic.__init__(self)
        ViewLogic.__init__(self, ImageViewer, self._changeImage)
        EditLogic.__init__(self, ImageEditor, self.currentImage)

        self._currentImage = None
        self._readOnly = True
        self._imageName = None

        self._popup = None
        self._popupLabel = QtGui.QLabel()

    def createPopup(self):
        p = self._popup = QtGui.QMainWindow(FX.mainWindow)
        # p.setFixedSize(512, 512)
        p.setWindowTitle('Zoom Image')
        p.setCentralWidget(self._popupLabel)
        p.hide()

    def deregister(self, remove=True):
        EnableLogic.deregister(self, remove)
        ViewLogic.deregister(self, remove)
        EditLogic.deregister(self, remove)

    def register(self):
        EnableLogic.register(self)
        ViewLogic.register(self)
        EditLogic.register(self)

    def _resolve(self, value):
        self._changeImage(value)

    def setEnabled(self, enabled):
        self._readOnly = not enabled
        QtGui.QLabel.setEnabled(self, True)

    def currentImage(self):
        return self._currentImage, self._imageName

    def mouseDoubleClickEvent(self, e):
        if self._readOnly or self._editor == None: 
            if self._popup == None:
                self.createPopup()

            self._popup.show()
            self._popup.raise_()
            return

        name = QtGui.QFileDialog.getOpenFileName(self, "Select new image", "", "Images (*.png *.jpg *.jpeg)")
        if name == None or len(name) == 0:
            return

        imgName = unicode(name)
        import os.path
        imgName = os.path.basename(imgName)
        self._imageName = imgName

        img = QtGui.QImage(name)
        self._updateImage(img)
        self._updatePopup(img)

        ba = QtCore.QByteArray()
        buffer = QtCore.QBuffer(ba)
        img.save(buffer, "PNG")

        self._currentImage = ba.data()
        self._editor._fieldChanged()

    def _updatePopup(self, img):
        p = QtGui.QPixmap()

        if img != None:
            p.convertFromImage(img)

        self._popupLabel.setPixmap(p)

    def _updateImage(self, img):
        p = QtGui.QPixmap()

        if img != None:
            p.convertFromImage(img)

        self.setPixmap(p)

    def _changeImage(self, d, name):
        self._imageName = name

        if d == None:
            self._updateImage(None)
            return

        d.addCallback(self._doChangeImage)

    def _doChangeImage(self, data):
        if data == None or len(data) == 0:
            self._updateImage(None)
            return

        self._currentImage = data

        img = QtGui.QImage.fromData(self._currentImage)
        self._updatePopup(img)

        img = img.scaled(self.width(), self.height(), QtCore.Qt.KeepAspectRatio, QtCore.Qt.SmoothTransformation)

        self._updateImage(img)



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

from ..baseWidget import *

from wallaby.pf.peer.payloadCallback import *

class PayloadLabel(QtGui.QLabel, BaseWidget):
    __metaclass__ = QWallabyMeta

    room   = Meta.property("string")
    pillow = Meta.property("string")

    def __init__(self, *args):
        QtGui.QLabel.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QLabel, *args)

        self._peer = None

    def deregister(self, remove=False):
        if self._peer: self._peer.destroy(remove)
        self._peer = None

    def register(self):
        self._peer = PayloadCallback(self.room, self.pillow, self.setText)




# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

from ..baseWidget import *
from wallaby.pf.peer.stack import *

class Controller(QtGui.QWidget, BaseWidget):
    __metaclass__ = QWallabyMeta

    room = Meta.property("string")

    def __init__(self, *args):
        QtGui.QWidget.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QWidget, *args)
        self._stack = None

        self._layerNames = {}
        self._layerNames['root'] = self

        self._layers = []
        self._layers.append(self)

    def register(self):
        self._stack = Stack(self.room, self)

        for layer in self._layers:
            if layer != self:
                layer.register()

    def deregister(self, remove=False):
        if self._stack != None: self._stack.destroy(remove)
        self._stack = None

        for layer in self._layers:
            if layer != self:
                layer.deregister(remove)

    # def resizeEvent(self, event):
    #     QtGui.QWidget.resizeEvent(self, event)

    #     for layer in self._layers:
    #         if layer != self:
    #             layer.resize(self.width(), self.height())

    def paintEvent(self, event):
        painter = QtGui.QPainter(self)
        painter.setBrush(QtGui.QColor(255,0,0,255))
        r = QtCore.QRectF(0,0,self.width(),self.height())
        painter.drawRect(r)

    def addLayer(self, layer, setZeroContentsMargins = True, group=None, hidden=False):
        name = layer._name
        if name == None: return

        topLayer = self._layers[len(self._layers)-1]
    
        l = QtGui.QVBoxLayout(topLayer)

        if setZeroContentsMargins == True:
            l.setContentsMargins(0,0,0,0)

        l.addWidget(layer)
        layer._controller = self

        self._layerNames[name] = layer
        self._layers.append(layer)
        layer._group = group

        if hidden:
            self.hideLayer(name)
        else:
            self.showLayer(name)

    def showLayer(self, name):
        currLayer = None
        for layer in self._layers:
            if layer != self:
                if layer._name == name:
                    currLayer = layer
                    break
        for layer in self._layers:
            if layer != self and layer._group != None and currLayer != None:
                if layer._group == currLayer._group and layer._name != name:
                    layer.setDisplayingContent(False)

        self._setLayerDisplayingContent(name, True)

    def hideLayer(self, name):
        self._setLayerDisplayingContent(name, False)

    def _setLayerDisplayingContent(self, name, displayingContent):
        if name == None: return

        if name in self._layerNames:
            layer = self._layerNames[name]
            if layer: layer.setDisplayingContent(displayingContent)

            self.update() #trigger repaint

    def enableLayer(self, name):
        self._setLayerUserinteractionEnabled(True)

    def disableLayer(self, name):
        self._setLayerUserinteractionEnabled(False)

    def _setLayerUserinteractionEnabled(self, name, userinteractionEnabled):
        if name == None: return

        if name in self._layerNames:
            layer = self._layerNames[name]
            if layer: layer.setUserinteractionEnabled(userinteractionEnabled)

    def isVisible(self, name):
        for layer in self._layers:
            if layer != self:
                if layer._group == name or layer._name == name:
                    if layer._displayingContent:
                        return layer._displayingContent
        return False



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/dominik/Documents/Projekte/wallaby/wallaby-frontend-qt/wallaby/frontends/qt/widgets/prefs/prefController.ui'
#
# Created: Mon Nov 26 16:21:03 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_PrefController(object):
    def setupUi(self, PrefController):
        PrefController.setObjectName(_fromUtf8("PrefController"))
        PrefController.resize(506, 701)
        self.verticalLayout_2 = QtGui.QVBoxLayout(PrefController)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setMargin(0)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.widget_3 = QtGui.QWidget(PrefController)
        self.widget_3.setObjectName(_fromUtf8("widget_3"))
        self.horizontalLayout_2 = QtGui.QHBoxLayout(self.widget_3)
        self.horizontalLayout_2.setMargin(0)
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.label = QtGui.QLabel(self.widget_3)
        self.label.setObjectName(_fromUtf8("label"))
        self.horizontalLayout_2.addWidget(self.label)
        self.presets = ComboBox(self.widget_3)
        self.presets.setMinimumSize(QtCore.QSize(200, 0))
        self.presets.setEditable(True)
        self.presets.setInsertPolicy(QtGui.QComboBox.NoInsert)
        self.presets.setObjectName(_fromUtf8("presets"))
        self.horizontalLayout_2.addWidget(self.presets)
        self.loadPresetButton = ActionButton(self.widget_3)
        self.loadPresetButton.setObjectName(_fromUtf8("loadPresetButton"))
        self.horizontalLayout_2.addWidget(self.loadPresetButton)
        self.savePresetButton = ActionButton(self.widget_3)
        self.savePresetButton.setObjectName(_fromUtf8("savePresetButton"))
        self.horizontalLayout_2.addWidget(self.savePresetButton)
        self.verticalLayout_2.addWidget(self.widget_3)
        self.scrollArea = QtGui.QScrollArea(PrefController)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName(_fromUtf8("scrollArea"))
        self.scrollAreaWidgetContents = QtGui.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 504, 599))
        self.scrollAreaWidgetContents.setObjectName(_fromUtf8("scrollAreaWidgetContents"))
        self.verticalLayout = QtGui.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setMargin(0)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.content = QtGui.QWidget(self.scrollAreaWidgetContents)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.content.sizePolicy().hasHeightForWidth())
        self.content.setSizePolicy(sizePolicy)
        self.content.setObjectName(_fromUtf8("content"))
        self.verticalLayout.addWidget(self.content)
        self.widget_2 = QtGui.QWidget(self.scrollAreaWidgetContents)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(100)
        sizePolicy.setHeightForWidth(self.widget_2.sizePolicy().hasHeightForWidth())
        self.widget_2.setSizePolicy(sizePolicy)
        self.widget_2.setObjectName(_fromUtf8("widget_2"))
        self.verticalLayout.addWidget(self.widget_2)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout_2.addWidget(self.scrollArea)
        self.widget = QtGui.QWidget(PrefController)
        self.widget.setObjectName(_fromUtf8("widget"))
        self.horizontalLayout = QtGui.QHBoxLayout(self.widget)
        self.horizontalLayout.setMargin(0)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.prefApplyButton = ActionButton(self.widget)
        self.prefApplyButton.setObjectName(_fromUtf8("prefApplyButton"))
        self.horizontalLayout.addWidget(self.prefApplyButton)
        self.prefRollbackButton = ActionButton(self.widget)
        self.prefRollbackButton.setObjectName(_fromUtf8("prefRollbackButton"))
        self.horizontalLayout.addWidget(self.prefRollbackButton)
        self.label_2 = QtGui.QLabel(self.widget)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.horizontalLayout.addWidget(self.label_2)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.prefSelect = QtGui.QComboBox(self.widget)
        self.prefSelect.setMinimumSize(QtCore.QSize(150, 0))
        self.prefSelect.setEditable(False)
        self.prefSelect.setInsertPolicy(QtGui.QComboBox.InsertAtBottom)
        self.prefSelect.setFrame(True)
        self.prefSelect.setObjectName(_fromUtf8("prefSelect"))
        self.horizontalLayout.addWidget(self.prefSelect)
        self.prefAdd = QtGui.QPushButton(self.widget)
        self.prefAdd.setMaximumSize(QtCore.QSize(32, 32))
        self.prefAdd.setText(_fromUtf8(""))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/images/add.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.prefAdd.setIcon(icon)
        self.prefAdd.setIconSize(QtCore.QSize(24, 24))
        self.prefAdd.setFlat(True)
        self.prefAdd.setObjectName(_fromUtf8("prefAdd"))
        self.horizontalLayout.addWidget(self.prefAdd)
        self.verticalLayout_2.addWidget(self.widget)

        self.retranslateUi(PrefController)
        QtCore.QMetaObject.connectSlotsByName(PrefController)

    def retranslateUi(self, PrefController):
        PrefController.setWindowTitle(QtGui.QApplication.translate("PrefController", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("PrefController", "Presets:", None, QtGui.QApplication.UnicodeUTF8))
        self.loadPresetButton.setText(QtGui.QApplication.translate("PrefController", "Load", None, QtGui.QApplication.UnicodeUTF8))
        self.savePresetButton.setText(QtGui.QApplication.translate("PrefController", "Save", None, QtGui.QApplication.UnicodeUTF8))
        self.prefApplyButton.setText(QtGui.QApplication.translate("PrefController", "apply", None, QtGui.QApplication.UnicodeUTF8))
        self.prefRollbackButton.setText(QtGui.QApplication.translate("PrefController", "cancel", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("PrefController", "Preference sheets", None, QtGui.QApplication.UnicodeUTF8))
        self.prefAdd.setToolTip(QtGui.QApplication.translate("PrefController", "Add", None, QtGui.QApplication.UnicodeUTF8))

from wallaby.frontends.qt.widgets.combo.comboBox import ComboBox
from wallaby.frontends.qt.widgets.buttons.actionButton import ActionButton
from . import resource_rc



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

from ..baseWidget import *
from wallaby.frontends.qt.models.multiViewTableModel import *
from wallaby.pf.peer.actionButtonPeer import *
from imageDelegate import *

class TreeView(QtGui.QTreeView, BaseWidget):
    __metaclass__ = QWallabyMeta

    room    = Meta.property("string")
    dbColumns  = Meta.property("list")
    tab        = Meta.property("string")

    sortable   = Meta.property("list")

    view           = Meta.property("string")
    viewArguments  = Meta.property("dict")
    viewIdentifier = Meta.property("string")
    dataView       = Meta.property("bool")

    doubleClickPillow   = Meta.property("string")
    doubleClickFeathers = Meta.property("string")
    doubleClickTab      = Meta.property("string")

    keyCodePillows     = Meta.property("list")

    multiSelect = Meta.property("bool")

    def __init__(self, parent = None):
        QtGui.QTreeView.__init__(self, parent)
        BaseWidget.__init__(self, QtGui.QTreeView)

        self.setSortingEnabled(False)
        self._sortable = {}
        self._columns = []
        self._labels = []
        self._keyCodePillows = {}

        self._ignoreClick = False

        self.header().setStretchLastSection(True)

        self._popup = None

        self._peer = None
        self._model = None

        self.header().setClickable(True)
        self.header().sectionClicked.connect(self._columnClicked)

        self.setAlternatingRowColors(True)
        self.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)

        self.clicked.connect(self._itemClicked)
        self.doubleClicked.connect(self._itemDoubleClicked)

    def _columnClicked(self, col):
        if col >= len(self._columns):
            return

        header = self.header()

        order = 0

        if self._columns[col] in self._sortable:
            order = self._sortable[self._columns[col]]
            if header.sortIndicatorSection() == col:
                order = (order + 1) % 2
            else:
                order = 0
        else:
            if "__default__" in self._sortable:
                order = (self._sortable["__default__"] + 1) % 2
            else:
                order = 0
            col = -1

        header.setSortIndicator(col, order)

        if col > -1:
            self._sortable[self._columns[col]] = order
        else:
            self._sortable["__default__"] = order

        self.model().sort(col, order)

    def _itemClicked(self, index):
        if self._ignoreClick:
            self._ignoreClick = False
            return

        self.resetSelection()

    def keyPressEvent(self, event):
        QtGui.QTableView.keyPressEvent(self, event)        

        if event.modifiers() & QtCore.Qt.ControlModifier and event.key() in self._keyCodePillows:
            pillow, feathers = self._keyCodePillows[event.key()]
            House.get(self.room).throw(pillow, feathers)
        else:
            self.resetSelection()

    def selectionChanged(self, selected, deselected):
        QtGui.QTableView.selectionChanged(self, selected, deselected)

        if self.multiSelect:
            sel = self.selectionModel().selectedIndexes()
            selection = set() 
            for s in sel:
                selection.add(s.row())

            self.model().doMultiSelect(selection)

    def resetSelection(self):
        sel = self.selectionModel().selectedIndexes()
        if len(sel) == 0:
            self.model().doSelect(None, None)
        else:
            self.model().doSelect(sel[0].row(), self.tab)

    def _itemDoubleClicked(self, index):
        # FIXME
        self._ignoreClick = True

        if self._peer:
            self._peer.buttonClicked()

    def deregister(self, remove=False):
        if self._model: self._model.destroyPeers(remove)
        if self._peer: self._peer.destroy(remove)
        self._peer = None

    def register(self):
        for c in self.sortable:
            self._sortable[c] = 0

        if len(self._sortable) > 0:
            self.header().setSortIndicatorShown(True)

        if self.multiSelect:
            self.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)

        self._columns, self._labels = FX.splitList(self.dbColumns, ':')
        self._types, _, self._columns = FX.extractType(self._columns)

        if self.keyCodePillows != None:
            for kca in self.keyCodePillows:
                kca = unicode(kca)
                code, pillow = kca.split(':')
                feathers = None
                if ',' in pillow:
                    feathers = pillow.split(',')
                    pillow = feathers.pop(0)

                code = eval("QtCore.Qt." + code)

                self._keyCodePillows[code] = (pillow, feather)

        col = 0
        for t in self._types:
            if isinstance(t, dict):
                for k, v in t.items():
                    if len(v) > 0 and v[0] == ':':
                        self.setItemDelegateForColumn(col, ImageDelegate())

            col = col + 1

        if self.doubleClickPillow != None:
            self._peer = ActionButtonPeer(self.room, self.doubleClickPillow, self.doubleClickFeathers, tab=self.doubleClickTab)

        self._model = MultiViewTableModel(self, self.dbColumns, roomName=self.room, view=self.view, viewIdentifier=self.viewIdentifier, viewArguments=self.viewArguments, dataView=self.dataView)
        self.setModel(self._model)




# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

from ..baseWidget import *
from ..logics import *

from wallaby.pf.peer.viewer import *

class Progress(QtGui.QProgressBar, BaseWidget, EnableLogic, ViewLogic):
    __metaclass__ = QWallabyMeta

    min = Meta.property("int")
    max = Meta.property("int")

    def __init__(self, *args):
        QtGui.QProgressBar.__init__(self, *args)
        BaseWidget.__init__(self, *args)
        EnableLogic.__init__(self)
        ViewLogic.__init__(self, Viewer, self._setText)

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        ViewLogic.deregister(self, remove)

    def register(self):
        EnableLogic.register(self)
        ViewLogic.register(self)

        if self.min: self.setMinimum(self.min)
        if self.max: self.setMaximum(self.max)

    def _setText(self, text):
        try:
            i = int(text)
        except ValueError:
            i = 0

        self.setValue(i)

    def conflict(self, doc):
        pass



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

import sys

from ..baseWidget import *
from ..logics import *

from wallaby.pf.peer.viewer import *
from wallaby.pf.peer.editor import *

class ValueButton(QtGui.QPushButton, BaseWidget, EnableLogic, ViewLogic, EditLogic):
    __metaclass__ = QWallabyMeta

    value = Meta.property("int")

    def __init__(self, *args):
        QtGui.QPushButton.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QPushButton, *args)
        EnableLogic.__init__(self)
        ViewLogic.__init__(self, Viewer, self._setValue)
        EditLogic.__init__(self, Editor, self._getValue)

        self.released.connect(self._released)
        self.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.setCheckable(True)

        self._labelsSingleViewer = None

    def _setValue(self, value):
        self.setChecked(value == self.value)

    def _getValue(self):
        return self.value

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        ViewLogic.deregister(self, remove)
        EditLogic.deregister(self, remove)

        if self._labelsSingleViewer: self._labelsSingleViewer.destroy()
        self._labelsSingleViewer = None

    def register(self):
        EnableLogic.register(self)
        ViewLogic.register(self, raw=True)
        EditLogic.register(self)

        if self.path: self._labelsSingleViewer = Viewer(self.room, self._setLabels, self.path+"Labels", raw=True)

    def _setLabels(self, labels):
        if labels and len(labels) > self.value:
            self.setText(labels[self.value])

    def _released(self):
        if not self.isChecked():
            self.setChecked(True)
        else:
            if self._editor: self._editor._fieldChanged()



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

import sys

from ..baseWidget import *
from ..logics import *

from wallaby.pf.peer.viewer import *
from wallaby.pf.peer.editor import *

class PlainTextEdit(QtGui.QPlainTextEdit, BaseWidget, EnableLogic, ViewLogic, EditLogic):
    __metaclass__ = QWallabyMeta

    isJson  = Meta.property("bool", default=False)
    throwWhileEditing = Meta.property("bool", extended=True, default=True)

    def __init__(self, *args):
        QtGui.QPlainTextEdit.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QPlainTextEdit, *args)
        EnableLogic.__init__(self)
        ViewLogic.__init__(self, Viewer, self._setValue)
        EditLogic.__init__(self, Editor, self._toPlainText)

        self._cb = None
        self._changed = True

        self._readOnly = self.isReadOnly()
        self.textChanged.connect(self._textChanged)

    def focusOutEvent ( self, event ):
        # self.setCursorWidth(0)

        if self._editor and self._changed:
            self._editor._fieldChanged()

        QtGui.QPlainTextEdit.focusOutEvent(self, event)

    def focusInEvent ( self, event ):
        # self.setCursorWidth(1)
        QtGui.QPlainTextEdit.focusInEvent(self, event)

    def _toPlainText (self):
        value = unicode(self.toPlainText())
       
        if self.isJson: 
            if value.startswith('{'):
                try:
                    import json
                    value = json.loads(value)
                except:           
                    pass
        else:
            return value

    def _setEnabled(self, enabled):
        if self._readOnly: return

        if self.hideIfDisabled:
            self.setVisible(enabled)
        else:
            self.setReadOnly(not enabled)

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        ViewLogic.deregister(self, remove)
        EditLogic.deregister(self, remove)

    def register(self):
        EnableLogic.register(self)
        ViewLogic.register(self, raw=True)
        EditLogic.register(self)

    def _resolve(self, value, **ka):
        if value == None:
            self._setValue( "")
        else:
            self._setValue(value)

        if self._editor: self._editor._resolve(**ka)

    def _setValue(self, val):
        if isinstance(val, (str, unicode)):
            val = unicode(val)
        else:
            if val != None:
                if self.isJson:
                    try:
                        import json
                        val = json.dumps(val, sort_keys=True, indent=4, separators=(',', ': '))
                    except:
                        val = unicode(val)
                else:
                    val = unicode(val)
            else:
                val = u""

        if self.toPlainText() != val:
            self._changed = False
            self.setPlainText(val)
            self._changed = True

    def _textChanged(self):
        if self.throwWhileEditing and self._editor and self._changed:
            self._editor._fieldChanged()
        self._changed = True

    def _docCB(self, document):
        print "doc", document
        print self.objectName()
        print "obj name", document.get(self.objectName())




# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/dominik/Documents/Projekte/wallaby/wallaby-frontend-qt/wallaby/frontends/qt/isheet/gbRoomAndPath.ui'
#
# Created: Sun Dec  2 12:36:08 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_GbRoomAndPath(object):
    def setupUi(self, GbRoomAndPath):
        GbRoomAndPath.setObjectName(_fromUtf8("GbRoomAndPath"))
        GbRoomAndPath.resize(94, 54)
        self.verticalLayout = QtGui.QVBoxLayout(GbRoomAndPath)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setMargin(0)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.widget = Widget(GbRoomAndPath)
        self.widget.setObjectName(_fromUtf8("widget"))
        self.verticalLayout.addWidget(self.widget)
        self.widget_2 = Widget(GbRoomAndPath)
        self.widget_2.setObjectName(_fromUtf8("widget_2"))
        self.verticalLayout.addWidget(self.widget_2)

        self.retranslateUi(GbRoomAndPath)
        QtCore.QMetaObject.connectSlotsByName(GbRoomAndPath)

    def retranslateUi(self, GbRoomAndPath):
        GbRoomAndPath.setWindowTitle(QtGui.QApplication.translate("GbRoomAndPath", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.widget.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-container-room", None, QtGui.QApplication.UnicodeUTF8))
        self.widget_2.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-container-path", None, QtGui.QApplication.UnicodeUTF8))

from wallaby.frontends.qt.widgets.containers.widget import Widget



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *
import sys

from ..baseWidget import *
from ..logics import *
from UI_numericLineEdit import *

from wallaby.pf.peer.viewer import *
from wallaby.pf.peer.editor import *

class NumericLineEdit(QtGui.QWidget, Ui_NumericLineEdit, BaseWidget, EnableLogic, ViewLogic, EditLogic):
    __metaclass__ = QWallabyMeta

    labelText    = Meta.property("string")
    unit         = Meta.property("string")
    hasSetButton = Meta.property("bool")
    decimals     = Meta.property("int")
    minimum      = Meta.property("double")
    maximum      = Meta.property("double")
    singleStep   = Meta.property("double")

    def __del__(self):
        print "DESTRUCT NUMERICLINEEDIT"

    def __init__(self, *args):
        QtGui.QWidget.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QWidget, *args)
        EnableLogic.__init__(self)
        ViewLogic.__init__(self, Viewer, self.setValue)
        EditLogic.__init__(self, Editor, self.getValue)

        self._ignorePalette = True

        self.setupUi(self)

        self._value = 0

        self.doubleSpinBox.editingFinished.connect(self.editingFinishedCB)
        self.pushButtonSet.clicked.connect(self.editingFinishedCB)

        self.registerPalette()

    def editingFinishedCB(self, *args):
        if self._editor:
            self._editor._fieldChanged()

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        ViewLogic.deregister(self, remove)
        EditLogic.deregister(self, remove)

    def register(self):
        EnableLogic.register(self)
        ViewLogic.register(self, raw=True)
        EditLogic.register(self, selectCallback=self._select)

        if self.decimals is not None: self.doubleSpinBox.setDecimals(self.decimals)
        if self.minimum is not None: self.doubleSpinBox.setMinimum(self.minimum)
        if self.maximum is not None: self.doubleSpinBox.setMaximum(self.maximum)
        if self.singleStep is not None: self.doubleSpinBox.setSingleStep(self.singleStep)

        if self.labelText and self.labelText != '':
            self.nameLabel.show()
            self.nameLabel.setText(self.labelText)
        else:
            self.nameLabel.hide()

        if self.unit and self.unit != '':
            self.unitLabel.show()
            self.unitLabel.setText(self.unit)
        else:
            self.unitLabel.hide()
        
        if self.hasSetButton:
            self.pushButtonSet.show()
        else:
            self.pushButtonSet.hide()
 
    def getValue(self):
        return self.doubleSpinBox.value()

    def _setFloat(self, value, cb):
        try:
            floatValue = float(value)
            if floatValue == None: floatValue = 0.0
            cb(floatValue)
        except:
            cb(0.0)

    def setValue(self, value):
        self._value = value
        self._setFloat(value, self.doubleSpinBox.setValue)

    def setMaxValue(self, value):
        self._setFloat(value, self.doubleSpinBox.setMaximum)
        self._setFloat(self._value, self.doubleSpinBox.setValue)

    def setMinValue(self, value):
        self._setFloat(value, self.doubleSpinBox.setMinimum)
        self._setFloat(self._value, self.doubleSpinBox.setValue)

    def setSingleStepValue(self, value):
        self._setFloat(value, self.doubleSpinBox.setSingleStep)
        self._setFloat(self._value, self.doubleSpinBox.setValue)
        
    def _select(self):
        self.doubleSpinBox.setFocus()
    
    def childPalette(self):
        return self.doubleSpinBox.palette()
    
    def setChildPalette(self, p):
        self.doubleSpinBox.setPalette(p)



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

class ProgressDelegate(QtGui.QStyledItemDelegate):
    def __init__(self, parent=None):
        QtGui.QStyledItemDelegate.__init__(self, parent)

    def paint(self, painter, option, index):
        obj = index.data(QtCore.Qt.DisplayRole)

        rect = option.rect

        painter.save()

        if not isinstance(obj, (float, int)):
            painter.drawText(rect, QtCore.Qt.AlignCenter, unicode(obj))
            painter.restore()
            return

        w = float(rect.width()-10)/100.0*float(obj)
        if w < 1.0: w = 1.0 
        if w > rect.width()-10: w = rect.width()-10

        margin = rect.height()/4
        painter.fillRect(rect.x()+5, rect.y()+margin, w, rect.height() - 2*margin, QtGui.QColor(0, 128, 128, 128))
        painter.drawText(rect, QtCore.Qt.AlignCenter, unicode(int(obj)) + u" %")
        painter.restore()

class ImageDelegate(QtGui.QStyledItemDelegate):
    def __init__(self, parent=None):
        QtGui.QStyledItemDelegate.__init__(self, parent)

    def paint(self, painter, option, index):
        # obj = index.data(QtCore.Qt.DisplayRole).toPyObject()
        obj = index.data(QtCore.Qt.DisplayRole)

        rect = option.rect
        # rect.adjust(rect.width()/3, 0, -rect.width()/3, 0)

        lst = []
        isList = False

        painter.save()

        maxHeight = 0

        if isinstance(obj, list):
            lst = []
            isList = True
            for o in obj:
                if isinstance(o, QtGui.QPixmap):
                    if o.height() > maxHeight: maxHeight = o.height()
                    lst.append(o)

        elif isinstance(obj, QtGui.QPixmap):
            lst.append(obj)
            if obj.height() > maxHeight: maxHeight = obj.height()
        else:
            if isinstance(obj, QtCore.QPyNullVariant): obj = "-"
            painter.drawText(rect, QtCore.Qt.AlignTop | QtCore.Qt.AlignHCenter | QtCore.Qt.TextWordWrap, unicode(obj))
            painter.restore()
            return

        if maxHeight == 0: 
            painter.restore()
            return

        if len(lst) == 0: 
            painter.restore()
            return

        if maxHeight > rect.height():
            ratio = float(rect.height()-2)/float(maxHeight)
        else:
            ratio = 1.0

        dx = lst[0].width()
        width = dx*len(lst)

        if not isList and width > rect.width():
            ratio2 = float(rect.width()-2)/float(width)
            if ratio2 < ratio: ratio = ratio2

        dx = int(ratio*float(dx))

        x = rect.x() # + rect.width()/2.0-width/2.0
        y = rect.y()
        i = 0

        lmargin = rect.height()/4

        while x < rect.x():
            i += 1
            x += dx

        while i < len(lst) and x + dx < rect.x() + rect.width():
            pixmap = lst[i]
            if pixmap.width() == 0 or pixmap.height() == 0: 
                i += 1
                continue
            if ratio < 1.0:
                pixmap = pixmap.scaled(QtCore.QSize(int(float(pixmap.width())*ratio), int(float(pixmap.height())*ratio)), QtCore.Qt.KeepAspectRatio)

            if not isList:
                painter.drawPixmap(x + option.rect.width()/2-pixmap.width()/2, y+option.rect.height()/2-pixmap.height()/2, pixmap)
            else:
                painter.drawPixmap(x, y, pixmap)

            if i < len(lst)-1:
                painter.drawLine(x+dx, y+lmargin, x+dx, y+rect.height()-lmargin)

            x += dx
            i += 1

        painter.restore()



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/dominik/Documents/Projekte/wallaby/wallaby-frontend-qt/wallaby/frontends/qt/isheet/iDefault.ui'
#
# Created: Tue Nov 27 12:58:17 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_IDefault(object):
    def setupUi(self, IDefault):
        IDefault.setObjectName(_fromUtf8("IDefault"))
        IDefault.resize(400, 300)
        self.verticalLayout = QtGui.QVBoxLayout(IDefault)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.label = QtGui.QLabel(IDefault)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName(_fromUtf8("label"))
        self.verticalLayout.addWidget(self.label)

        self.retranslateUi(IDefault)
        QtCore.QMetaObject.connectSlotsByName(IDefault)

    def retranslateUi(self, IDefault):
        IDefault.setWindowTitle(QtGui.QApplication.translate("IDefault", "Default Tab", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("IDefault", "Please select a widget.", None, QtGui.QApplication.UnicodeUTF8))




# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

from ..baseWidget import *
from ..logics import *

from wallaby.pf.peer.viewer import *
from wallaby.pf.peer.payloadCallback import *

class WebView(QtWebKit.QWebView, BaseWidget, EnableLogic, ViewLogic, TriggeredPillowsLogic):
    __metaclass__ = QWallabyMeta

    markdown = Meta.property('bool')
    codeHighlight = Meta.property('bool')
    allowExternalLinks = Meta.property('bool', default=False)
    useValueAsUrl = Meta.property('bool', default=False)

    identifier = Meta.property('string')

    triggers = Meta.property("list", readOnly=True, default=["", "double-clicked"])

    def __init__(self, *args):
        QtWebKit.QWebView.__init__(self, *args)
        BaseWidget.__init__(self, QtWebKit.QWebView, *args)
        EnableLogic.__init__(self)
        ViewLogic.__init__(self, Viewer, self._setText)
        TriggeredPillowsLogic.__init__(self)
        self.linkClicked.connect(self._linkActivated)

        self._backPeer = None
        self._forwardPeer = None
        self._reloadPeer = None

    def mouseDoubleClickEvent(self, e):
        self.trigger("double-clicked")

    def _linkActivated(self, url):
        link = unicode(url.toString())
        if link.startswith('wallaby://'):
            link = link.replace('wallaby://', '')
            args = None
            if '/' in link:
                args = link.split('/')
                _ = args.pop(0)
                link = args.pop(0)

                if len(args) == 1:
                    args = args[0]

            House.get(self.room).throw(link, args)
        else:
            if self.allowExternalLinks:
                self.load(url)
            else:
                QtGui.QDesktopServices.openUrl(QtCore.QUrl(link))

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        ViewLogic.deregister(self, remove)
        TriggeredPillowsLogic.deregister(self, remove)

        for peer in (self._backPeer, self._forwardPeer, self._reloadPeer):
            if peer: peer.destroy()

        self._backPeer = None
        self._forwardPeer = None
        self._reloadPeer = None

    def register(self):
        EnableLogic.register(self)
        ViewLogic.register(self)
        TriggeredPillowsLogic.register(self)

        self._backPeer = PayloadCallback(self.room, "WebViewer.In.Back", self._back)
        self._forwardPeer = PayloadCallback(self.room, "WebViewer.In.Forward", self._forward)
        self._reloadPeer = PayloadCallback(self.room, "WebViewer.In.Reload", self._reload)

    def _back(self, ident):
        if self.identifier is None or ident == self.identifier: 
            self.back() 

    def _forward(self, *args):
        if self.identifier is None or ident == self.identifier: 
            self.forward() 

    def _reload(self, *args):
        if self.identifier is None or ident == self.identifier: 
            self.reload() 

    def _setText(self, text):
        if self.useValueAsUrl:
            try:
                url = QtCore.QUrl(unicode(text))
                self.load(url) 
                return
            except: pass

        if self.markdown:
            import markdown
            if self.codeHighlight:
                text = markdown.markdown(text, ['fenced_code', 'codehilite'])
                prefix = """
<html><head>
<style type="text/css">
.hll { background-color: #ffffcc }
.c { color: #999988; font-style: italic } /* Comment */
.err { color: #a61717; background-color: #e3d2d2 } /* Error */
.k { color: #000000; font-weight: bold } /* Keyword */
.o { color: #000000; font-weight: bold } /* Operator */
.cm { color: #999988; font-style: italic } /* Comment.Multiline */
.cp { color: #999999; font-weight: bold; font-style: italic } /* Comment.Preproc */
.c1 { color: #999988; font-style: italic } /* Comment.Single */
.cs { color: #999999; font-weight: bold; font-style: italic } /* Comment.Special */
.gd { color: #000000; background-color: #ffdddd } /* Generic.Deleted */
.ge { color: #000000; font-style: italic } /* Generic.Emph */
.gr { color: #aa0000 } /* Generic.Error */
.gh { color: #999999 } /* Generic.Heading */
.gi { color: #000000; background-color: #ddffdd } /* Generic.Inserted */
.go { color: #888888 } /* Generic.Output */
.gp { color: #555555 } /* Generic.Prompt */
.gs { font-weight: bold } /* Generic.Strong */
.gu { color: #aaaaaa } /* Generic.Subheading */
.gt { color: #aa0000 } /* Generic.Traceback */
.kc { color: #000000; font-weight: bold } /* Keyword.Constant */
.kd { color: #000000; font-weight: bold } /* Keyword.Declaration */
.kn { color: #000000; font-weight: bold } /* Keyword.Namespace */
.kp { color: #000000; font-weight: bold } /* Keyword.Pseudo */
.kr { color: #000000; font-weight: bold } /* Keyword.Reserved */
.kt { color: #445588; font-weight: bold } /* Keyword.Type */
.m { color: #009999 } /* Literal.Number */
.s { color: #d01040 } /* Literal.String */
.na { color: #008080 } /* Name.Attribute */
.nb { color: #0086B3 } /* Name.Builtin */
.nc { color: #445588; font-weight: bold } /* Name.Class */
.no { color: #008080 } /* Name.Constant */
.nd { color: #3c5d5d; font-weight: bold } /* Name.Decorator */
.ni { color: #800080 } /* Name.Entity */
.ne { color: #990000; font-weight: bold } /* Name.Exception */
.nf { color: #990000; font-weight: bold } /* Name.Function */
.nl { color: #990000; font-weight: bold } /* Name.Label */
.nn { color: #555555 } /* Name.Namespace */
.nt { color: #000080 } /* Name.Tag */
.nv { color: #008080 } /* Name.Variable */
.ow { color: #000000; font-weight: bold } /* Operator.Word */
.w { color: #bbbbbb } /* Text.Whitespace */
.mf { color: #009999 } /* Literal.Number.Float */
.mh { color: #009999 } /* Literal.Number.Hex */
.mi { color: #009999 } /* Literal.Number.Integer */
.mo { color: #009999 } /* Literal.Number.Oct */
.sb { color: #d01040 } /* Literal.String.Backtick */
.sc { color: #d01040 } /* Literal.String.Char */
.sd { color: #d01040 } /* Literal.String.Doc */
.s2 { color: #d01040 } /* Literal.String.Double */
.se { color: #d01040 } /* Literal.String.Escape */
.sh { color: #d01040 } /* Literal.String.Heredoc */
.si { color: #d01040 } /* Literal.String.Interpol */
.sx { color: #d01040 } /* Literal.String.Other */
.sr { color: #009926 } /* Literal.String.Regex */
.s1 { color: #d01040 } /* Literal.String.Single */
.ss { color: #990073 } /* Literal.String.Symbol */
.bp { color: #999999 } /* Name.Builtin.Pseudo */
.vc { color: #008080 } /* Name.Variable.Class */
.vg { color: #008080 } /* Name.Variable.Global */
.vi { color: #008080 } /* Name.Variable.Instance */
.il { color: #009999 } /* Literal.Number.Integer.Long */
</style>
</head>
<body>
                """

                text = prefix + text + "</body></html>"
            else:
                text = markdown.markdown(text, ['fenced_code'])

        self.setHtml(text)
        self.page().setLinkDelegationPolicy(QtWebKit.QWebPage.DelegateAllLinks)



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

from ..logics import *
from ..baseWidget import *

from wallaby.pf.peer.viewer import *
from wallaby.pf.peer.editor import *

class Slider(QtGui.QSlider, BaseWidget, EnableLogic, ViewLogic, EditLogic):
    __metaclass__ = QWallabyMeta

    min = Meta.property("int")
    max = Meta.property("int")

    def __init__(self, *args):
        QtGui.QSlider.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QSlider, *args)
        EnableLogic.__init__(self)
        ViewLogic.__init__(self, Viewer, self.setVal)
        EditLogic.__init__(self, Editor, self.sliderPosition)

        self.valueChanged.connect(self.changedValue)
        self._pause = False

    def register(self):
        EnableLogic.register(self)
        ViewLogic.register(self, raw=True)
        EditLogic.register(self)

        if self.min: self.setMinimum(self.min)
        if self.max: self.setMaximum(self.max)

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        ViewLogic.deregister(self, remove)
        EditLogic.deregister(self, remove)

    def setVal(self, value):
        try:
            valueInt = int(value)
        except:
            valueInt = 0

        self._pause = True
        self.setValue(valueInt)
        self._pause = False

    def changedValue(self):
        if self._editor and not self._pause: self._editor._fieldChanged()



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/dominik/Documents/Projekte/wallaby/wallaby-frontend-qt/wallaby/frontends/qt/widgets/edits/numericLineEdit.ui'
#
# Created: Mon Nov 19 14:07:43 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_NumericLineEdit(object):
    def setupUi(self, NumericLineEdit):
        NumericLineEdit.setObjectName(_fromUtf8("NumericLineEdit"))
        NumericLineEdit.resize(220, 37)
        self.horizontalLayout = QtGui.QHBoxLayout(NumericLineEdit)
        self.horizontalLayout.setSpacing(5)
        self.horizontalLayout.setMargin(1)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.nameLabel = QtGui.QLabel(NumericLineEdit)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.MinimumExpanding, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.nameLabel.sizePolicy().hasHeightForWidth())
        self.nameLabel.setSizePolicy(sizePolicy)
        self.nameLabel.setMinimumSize(QtCore.QSize(50, 0))
        self.nameLabel.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.nameLabel.setIndent(2)
        self.nameLabel.setObjectName(_fromUtf8("nameLabel"))
        self.horizontalLayout.addWidget(self.nameLabel)
        self.doubleSpinBox = QtGui.QDoubleSpinBox(NumericLineEdit)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.doubleSpinBox.sizePolicy().hasHeightForWidth())
        self.doubleSpinBox.setSizePolicy(sizePolicy)
        self.doubleSpinBox.setMinimumSize(QtCore.QSize(100, 0))
        self.doubleSpinBox.setMaximumSize(QtCore.QSize(100, 16777215))
        self.doubleSpinBox.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.doubleSpinBox.setReadOnly(False)
        self.doubleSpinBox.setButtonSymbols(QtGui.QAbstractSpinBox.UpDownArrows)
        self.doubleSpinBox.setAccelerated(False)
        self.doubleSpinBox.setPrefix(_fromUtf8(""))
        self.doubleSpinBox.setSuffix(_fromUtf8(""))
        self.doubleSpinBox.setMaximum(99999.99)
        self.doubleSpinBox.setProperty("value", 99999.99)
        self.doubleSpinBox.setObjectName(_fromUtf8("doubleSpinBox"))
        self.horizontalLayout.addWidget(self.doubleSpinBox)
        self.unitLabel = QtGui.QLabel(NumericLineEdit)
        self.unitLabel.setMinimumSize(QtCore.QSize(30, 0))
        self.unitLabel.setMaximumSize(QtCore.QSize(30, 16777215))
        self.unitLabel.setObjectName(_fromUtf8("unitLabel"))
        self.horizontalLayout.addWidget(self.unitLabel)
        self.pushButtonSet = QtGui.QPushButton(NumericLineEdit)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButtonSet.sizePolicy().hasHeightForWidth())
        self.pushButtonSet.setSizePolicy(sizePolicy)
        self.pushButtonSet.setMaximumSize(QtCore.QSize(30, 16777215))
        self.pushButtonSet.setBaseSize(QtCore.QSize(0, 0))
        self.pushButtonSet.setObjectName(_fromUtf8("pushButtonSet"))
        self.horizontalLayout.addWidget(self.pushButtonSet)

        self.retranslateUi(NumericLineEdit)
        QtCore.QMetaObject.connectSlotsByName(NumericLineEdit)

    def retranslateUi(self, NumericLineEdit):
        NumericLineEdit.setWindowTitle(QtGui.QApplication.translate("NumericLineEdit", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.nameLabel.setText(QtGui.QApplication.translate("NumericLineEdit", "X", None, QtGui.QApplication.UnicodeUTF8))
        self.unitLabel.setText(QtGui.QApplication.translate("NumericLineEdit", "nm", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButtonSet.setText(QtGui.QApplication.translate("NumericLineEdit", "Set", None, QtGui.QApplication.UnicodeUTF8))




# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

import sys

from ..baseWidget import *
from ..logics import *

from wallaby.pf.peer.booleanViewer import *
from wallaby.pf.peer.booleanEditor import *

class PushButton(QtGui.QPushButton, BaseWidget, EnableLogic, ViewLogic, EditLogic, TriggeredPillowsLogic):
    __metaclass__ = QWallabyMeta

    def __init__(self, *args):
        QtGui.QPushButton.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QPushButton, *args)
        EnableLogic.__init__(self)
        ViewLogic.__init__(self, BooleanViewer, self.setChecked)
        EditLogic.__init__(self, BooleanEditor, self.isChecked)

        self.released.connect(self._released)
        self.setFocusPolicy(QtCore.Qt.StrongFocus)

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        ViewLogic.deregister(self, remove)
        EditLogic.deregister(self, remove)

    def register(self):
        EnableLogic.register(self)
        ViewLogic.register(self)
        EditLogic.register(self)

    def _released(self):
        if self._editor: self._editor._fieldChanged()
 



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

from peer import *
from viewer import *
from wallaby.backends.couchdb import *
from wallaby.qt_combat import *

class OpenInBrowser(Peer):
    Open = Pillow.In

    Receiving = [
        Viewer.In.Document
    ]

    def __init__(self, room, databaseName=None):
        Peer.__init__(self, room)

        self._id     = None
        self._dbName = databaseName

        self._catch(OpenInBrowser.In.Open, self._open)
        self._catch(Viewer.In.Document, self._docArrived)

        self._doc = None

    def _open(self, pillow, feathers):
        if feathers != None and len(feathers) > 0:
            if feathers.startswith("/"):
                url = QtCore.QUrl(dbURL+feathers)
                QtGui.QDesktopServices.openUrl(url)
                return

            if '/' not in feathers and '.' in feathers and self._doc != None:
                url = self._doc.get(feathers) 
                if url != None:
                    self._open(None, url)
                return

            url = QtCore.QUrl(feathers)
            QtGui.QDesktopServices.openUrl(url)
            return

        if self._id != None:
            dbURL = Database.getURLForDatabase(self._dbName)
            if dbURL == None: return

            if self._dbName == None:
                self._dbName = Database.getDefaultDatabaseName()

            url = QtCore.QUrl(dbURL+'/_utils/document.html?'+self._dbName+'/'+self._id)
            QtGui.QDesktopServices.openUrl(url)

    def _docArrived(self, pillow, doc):
        self._doc = doc
        if doc != None:
            self._id = doc.documentID

    def initialize(self):
        pass



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './wallaby/frontends/qt/isheet/gbSource.ui'
#
# Created: Thu Dec  6 17:58:41 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_GbRoomAndPath(object):
    def setupUi(self, GbRoomAndPath):
        GbRoomAndPath.setObjectName(_fromUtf8("GbRoomAndPath"))
        GbRoomAndPath.resize(652, 89)
        self.gridLayout = QtGui.QGridLayout(GbRoomAndPath)
        self.gridLayout.setContentsMargins(0, 20, 0, 0)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.label = QtGui.QLabel(GbRoomAndPath)
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.label_2 = QtGui.QLabel(GbRoomAndPath)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1)
        self.LineEdit = LineEdit(GbRoomAndPath)
        self.LineEdit.setObjectName(_fromUtf8("LineEdit"))
        self.gridLayout.addWidget(self.LineEdit, 0, 1, 1, 1)
        self.LineEdit_2 = LineEdit(GbRoomAndPath)
        self.LineEdit_2.setObjectName(_fromUtf8("LineEdit_2"))
        self.gridLayout.addWidget(self.LineEdit_2, 2, 1, 1, 1)

        self.retranslateUi(GbRoomAndPath)
        QtCore.QMetaObject.connectSlotsByName(GbRoomAndPath)

    def retranslateUi(self, GbRoomAndPath):
        GbRoomAndPath.setWindowTitle(QtGui.QApplication.translate("GbRoomAndPath", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("GbRoomAndPath", "Document-id of source document", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("GbRoomAndPath", "Path to data (list of strings)", None, QtGui.QApplication.UnicodeUTF8))
        self.LineEdit.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-source", None, QtGui.QApplication.UnicodeUTF8))
        self.LineEdit_2.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-sourcepath", None, QtGui.QApplication.UnicodeUTF8))

from wallaby.frontends.qt.widgets.edits.lineEdit import LineEdit



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

import FX as FX
from twisted.python import log
from twisted.internet import defer, task
import sys, traceback, os, re, json
import socket
from platform import *

mineIcon = None
theirsIcon = None

configEditor = None

pLogo       = None
app         = None
mainWindow  = None

widgets  = []

def replaceValue(dct, val):
    val = FX.replaceValue(dct, val) 

    from qt_combat import QtGui, QtCore

    if len(val) > 0 and val[0] == ':':
        val = val.lower()
        val = QtGui.QPixmap(val)

    return val

def renderType(val, type, args, edit=False):
    val = FX.renderType(val, type, args, edit=edit, fct=replaceValue)

    from wallaby.qt_combat import QtCore
    if isinstance(val, QtCore.QPyNullVariant):
        val = "-"

    return val

def addShutdownCB(cb):
    shutdownCBs.append(cb)

def callShutdownCBs():
    for cb in shutdownCBs:
        cb()

def Property(function):
    keys = 'fget', 'fset', 'fdel'
    func_locals = {'doc':function.__doc__}
    def probeFunc(frame, event, arg):
        if event == 'return':
            locals = frame.f_locals
            func_locals.update(dict((k,locals.get(k)) for k in keys))
            sys.settrace(None)
        return probeFunc
    sys.settrace(probeFunc)
    function()
    return property(**func_locals)

def imp(name):
    try:
        mod = __import__(name)
        components = name.split('.')
        for comp in components[1:]:
            mod = getattr(mod, comp)
        return mod
    except Exception as e:
        if True or not isinstance(e, ImportError):
            import traceback 
            traceback.print_exc(file=sys.stdout)
        return None

def compile(path):
    from wallaby.frontends.qt.compile import compile
    for p in path:
        compile(p)

def info(*msg):
    log.msg(toMessage("INFO" ,msg), logLevel=INFO)
    
def warn(*msg):    
    log.msg(toMessage("WARNING" ,msg), logLevel=WARN)
        
def crit(*msg):    
    log.err(toMessage("CRITICAL" ,msg), logLevel=CRIT)
        
def debug(*msg):    
    print toMessage("DEBUG", msg) #TODO: remove
    if not logDebug:
        return

    log.msg(toMessage("DEBUG" , msg), logLevel=DEBUG)
    
def trace(*params):
    if not logTrace:
        return

    log.msg(toMessage("TRACE" , params), logLevel=TRACE)
    
    type, value, trace = sys.exc_info()   
    stack = traceback.extract_stack()
    call  = stack[len(stack)-2]
    msg   = "FILE: " + call[0] + "  LINE: " + str(call[1])  
    log.msg(msg, logLevel=TRACE)

#------------------------------------------------------------------------------
# Copyright (c) 2007, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD license.
# However, when used with the GPL version of PyQt the additional terms described in the PyQt GPL exception also apply

#
# Author: Riverbank Computing Limited
#------------------------------------------------------------------------------

""" Converts a QKeyEvent to a standardized "name".
"""

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

from wallaby.qt_combat import *

#-------------------------------------------------------------------------------
#  Constants:
#-------------------------------------------------------------------------------
 
# Mapping from PyQt keypad key names to Pyface key names.
keypad_map = {
    QtCore.Qt.Key_Enter:     'Enter',
    QtCore.Qt.Key_0:         'Numpad 0',
    QtCore.Qt.Key_1:         'Numpad 1',
    QtCore.Qt.Key_2:         'Numpad 2',
    QtCore.Qt.Key_3:         'Numpad 3',
    QtCore.Qt.Key_4:         'Numpad 4',
    QtCore.Qt.Key_5:         'Numpad 5',
    QtCore.Qt.Key_6:         'Numpad 6',
    QtCore.Qt.Key_7:         'Numpad 7',
    QtCore.Qt.Key_8:         'Numpad 8',
    QtCore.Qt.Key_9:         'Numpad 9',
    QtCore.Qt.Key_Asterisk:  'Multiply',
    QtCore.Qt.Key_Plus:      'Add',
    QtCore.Qt.Key_Comma:     'Separator',
    QtCore.Qt.Key_Minus:     'Subtract',
    QtCore.Qt.Key_Period:    'Decimal',
    QtCore.Qt.Key_Slash:     'Divide'
}
 
# Mapping from PyQt non-keypad key names to Pyface key names.
key_map = {
    QtCore.Qt.Key_0:         '0',
    QtCore.Qt.Key_1:         '1',
    QtCore.Qt.Key_2:         '2',
    QtCore.Qt.Key_3:         '3',
    QtCore.Qt.Key_4:         '4',
    QtCore.Qt.Key_5:         '5',
    QtCore.Qt.Key_6:         '6',
    QtCore.Qt.Key_7:         '7',
    QtCore.Qt.Key_8:         '8',
    QtCore.Qt.Key_9:         '9',
    QtCore.Qt.Key_A:         'A',
    QtCore.Qt.Key_B:         'B',
    QtCore.Qt.Key_C:         'C',
    QtCore.Qt.Key_D:         'D',
    QtCore.Qt.Key_E:         'E',
    QtCore.Qt.Key_F:         'F',
    QtCore.Qt.Key_G:         'G',
    QtCore.Qt.Key_H:         'H',
    QtCore.Qt.Key_I:         'I',
    QtCore.Qt.Key_J:         'J',
    QtCore.Qt.Key_K:         'K',
    QtCore.Qt.Key_L:         'L',
    QtCore.Qt.Key_M:         'M',
    QtCore.Qt.Key_N:         'N',
    QtCore.Qt.Key_O:         'O',
    QtCore.Qt.Key_P:         'P',
    QtCore.Qt.Key_Q:         'Q',
    QtCore.Qt.Key_R:         'R',
    QtCore.Qt.Key_S:         'S',
    QtCore.Qt.Key_T:         'T',
    QtCore.Qt.Key_U:         'U',
    QtCore.Qt.Key_V:         'V',
    QtCore.Qt.Key_W:         'W',
    QtCore.Qt.Key_X:         'X',
    QtCore.Qt.Key_Y:         'Y',
    QtCore.Qt.Key_Z:         'Z',
    QtCore.Qt.Key_Space:     'Space',
    QtCore.Qt.Key_Backspace: 'Backspace',
    QtCore.Qt.Key_Tab:       'Tab',
    QtCore.Qt.Key_Enter:     'Enter',
    QtCore.Qt.Key_Return:    'Return',
    QtCore.Qt.Key_Escape:    'Esc',
    QtCore.Qt.Key_Delete:    'Delete',
    QtCore.Qt.Key_Cancel:    'Cancel',
    QtCore.Qt.Key_Clear:     'Clear',
    QtCore.Qt.Key_Shift:     'Shift',
    QtCore.Qt.Key_Menu:      'Menu',
    QtCore.Qt.Key_Pause:     'Pause',
    QtCore.Qt.Key_PageUp:    'Page Up',
    QtCore.Qt.Key_PageDown:  'Page Down',
    QtCore.Qt.Key_End:       'End',
    QtCore.Qt.Key_Home:      'Home',
    QtCore.Qt.Key_Left:      'Left',
    QtCore.Qt.Key_Up:        'Up',
    QtCore.Qt.Key_Right:     'Right',
    QtCore.Qt.Key_Down:      'Down',
    QtCore.Qt.Key_Select:    'Select',
    QtCore.Qt.Key_Print:     'Print',
    QtCore.Qt.Key_Execute:   'Execute',
    QtCore.Qt.Key_Insert:    'Insert',
    QtCore.Qt.Key_Help:      'Help',
    QtCore.Qt.Key_F1:        'F1',
    QtCore.Qt.Key_F2:        'F2',
    QtCore.Qt.Key_F3:        'F3',
    QtCore.Qt.Key_F4:        'F4',
    QtCore.Qt.Key_F5:        'F5',
    QtCore.Qt.Key_F6:        'F6',
    QtCore.Qt.Key_F7:        'F7',
    QtCore.Qt.Key_F8:        'F8',
    QtCore.Qt.Key_F9:        'F9',
    QtCore.Qt.Key_F10:       'F10',
    QtCore.Qt.Key_F11:       'F11',
    QtCore.Qt.Key_F12:       'F12',
    QtCore.Qt.Key_F13:       'F13',
    QtCore.Qt.Key_F14:       'F14',
    QtCore.Qt.Key_F15:       'F15',
    QtCore.Qt.Key_F16:       'F16',
    QtCore.Qt.Key_F17:       'F17',
    QtCore.Qt.Key_F18:       'F18',
    QtCore.Qt.Key_F19:       'F19',
    QtCore.Qt.Key_F20:       'F20',
    QtCore.Qt.Key_F21:       'F21',
    QtCore.Qt.Key_F22:       'F22',
    QtCore.Qt.Key_F23:       'F23',
    QtCore.Qt.Key_F24:       'F24',
    QtCore.Qt.Key_NumLock:   'Num Lock',
    QtCore.Qt.Key_ScrollLock:'Scroll Lock'
}
 
#-------------------------------------------------------------------------------
#  Converts a keystroke event into a corresponding key name:
#-------------------------------------------------------------------------------
 
def key_event_to_name(event):
    """ Converts a keystroke event into a corresponding key name.
    """
    key_code = event.key()
    modifiers = event.modifiers()
    if modifiers & QtCore.Qt.KeypadModifier:
        key = keypad_map.get(key_code)
    else:
        key = None
    if key is None:
        key = key_map.get(key_code)
 
    name = ''
    if modifiers & QtCore.Qt.ControlModifier:
        name += 'Ctrl'
 
    if modifiers & QtCore.Qt.AltModifier:
        name += '-Alt' if name else 'Alt'
 
    if modifiers & QtCore.Qt.MetaModifier:
        name += '-Meta' if name else 'Meta'
 
    if modifiers & QtCore.Qt.ShiftModifier and ((name != '') or (key is not None and len(key) > 1)):
        name += '-Shift' if name else 'Shift'
 
    if key:
        if name:
            name += '-'
        name += key
    return name
    
#-------------------------------------------------------------------------------

def toMessage(prefix, list):
    msg = prefix
    for m in list:
        msg = msg + " " + str(m)
    return msg
    
def deferredSleep(seconds):
    d = defer.Deferred()

    from twisted.internet import reactor
    reactor.callLater(seconds, d.callback, seconds)
    return d

def splitPath(path):
    return path.split('.');

def deferredCall(*args):
    from twisted.internet import reactor
    return task.deferLater(reactor, 0, *args)

def icon():
    from wallaby.qt_combat import QtGui
    return QtGui.QIcon(logo())

def logo():
    global pLogo
    if pLogo: return pLogo

    from wallaby.qt_combat import QtGui

    _logo_16x16_xpm = [
    "16 16 75 1 ",
    "  c #DE0101",
    ". c #DF0101",
    "X c #E20101",
    "o c #E5180C",
    "O c #E61B0E",
    "+ c #E62417",
    "@ c #E72519",
    "# c #E7261A",
    "$ c #E72B1F",
    "% c #E73629",
    "& c #E83B31",
    "* c #EA4136",
    "= c #EA453A",
    "- c #EB4C42",
    "; c #EB4F46",
    ": c #EC4F46",
    "> c #EB5148",
    ", c #EB594E",
    "< c #EA5B52",
    "1 c #EC5E55",
    "2 c #EE635A",
    "3 c #ED655B",
    "4 c #EE665D",
    "5 c #EF665D",
    "6 c #ED6961",
    "7 c #EF746C",
    "8 c #F0766F",
    "9 c #F1837B",
    "0 c #F18881",
    "q c #F28982",
    "w c #F08C85",
    "e c #F18C86",
    "r c #F3938B",
    "t c #F39893",
    "y c #F39993",
    "u c #F49B95",
    "i c #F4A19B",
    "p c #F4A19D",
    "a c #F4A29C",
    "s c #F5A49F",
    "d c #F5A8A2",
    "f c #F5AAA6",
    "g c #F6ABA6",
    "h c #F6AAA7",
    "j c #F6AEAB",
    "k c #F6B4B0",
    "l c #F7B6B2",
    "z c #F8BBB7",
    "x c #F7BEBA",
    "c c #F8C5C3",
    "v c #F8C6C2",
    "b c #F9C6C3",
    "n c #F8C9C6",
    "m c #FACCC9",
    "M c #F9CECB",
    "N c #F9CFCC",
    "B c #FAD1CE",
    "V c #F9D2CF",
    "C c #F9D3D0",
    "Z c #FAD5D2",
    "A c #FBD8D5",
    "S c #F9DAD9",
    "D c #FCDAD9",
    "F c #FCDDDB",
    "G c #FBDEDC",
    "H c #FBDFDD",
    "J c #FCE6E5",
    "K c #FCE7E5",
    "L c #FDEAE8",
    "P c #FDEFED",
    "I c #FDEFEE",
    "U c #FDF0EE",
    "Y c #FEF4F4",
    "T c #FEFFFF",
    "R c gray100",
    "RRRRRRRRRRRRRRRR",
    "RRRRRRRH&aRRRRRR",
    "RRRRRRRR*X7RRRRR",
    "RRRRRRRRwuRRRRRR",
    "RRRRRRRw=fSRRRRR",
    "RRRRRRM$rt3RRRRR",
    "RRRRRRf9vAdHRRRR",
    "RRRRRRR%3luRRRRR",
    "RHRRRRf X0<RRRRR",
    "Ra6nA<  o3wRRRRR",
    "RRk>@+;+z7BRRRRR",
    "RRRRRRc;R93RRRRR",
    "RRRRRRMoPJaxJRRR",
    "RRRRRRB aRH;BRRR",
    "RRRRRRU,cRYjPRRR",
    "RRRRRRRMLRRRRRRR" ]
    
    pLogo = QtGui.QPixmap(_logo_16x16_xpm)
    return pLogo



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *
import sys

from ..baseWidget import *
from ..logics import *

from wallaby.pf.peer.dateViewer import *
from wallaby.pf.peer.dateEditor import *

class DateEdit(QtGui.QDateEdit, BaseWidget, EnableLogic, ViewLogic, EditLogic):
    __metaclass__ = QWallabyMeta

    defaultToNow = Meta.property("bool", extended=True)

    def __init__(self, *args):
        QtGui.QDateEdit.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QDateEdit, *args)
        EnableLogic.__init__(self)
        ViewLogic.__init__(self, DateViewer, self._setValue)
        EditLogic.__init__(self, DateEditor, self._value)

        self._changed = True

        self._readOnly = self.isReadOnly()
        self.dateChanged.connect(self._valueChanged)

    def _setEnabled(self, enabled):
        if self._readOnly: return

        if self.hideIfDisabled:
            self.setVisible(enabled)
        else:
            self.setReadOnly(not enabled)

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        ViewLogic.deregister(self, remove)
        EditLogic.deregister(self, remove)

    def register(self):
        EnableLogic.register(self)
        ViewLogic.register(self)
        EditLogic.register(self)

        if self.defaultToNow:
            d = QtCore.QDate.currentDate()
        else:
            d = QtCore.QDate(1900, 1, 1)
 

    def _resolve(self, value, **ka):
        self._setValue(value)
        if self._editor: self._editor._resolve(**ka)

    def _value(self):
        d = self.date()

        return [d.year(), d.month(), d.day()]

    def _valueChanged(self, val):
        if self._editor and self._changed:
            self._editor._fieldChanged()

    def _setValue(self, value):
        if value == None:
            if self.defaultToNow:
                d = QtCore.QDate.currentDate()
            else:
                d = QtCore.QDate(1900, 1, 1)
        else:
            if len(value) >= 3:
                d = QtCore.QDate(value[0], value[1], value[2])
            
        self._changed = False
        self.setDate(d)
        self._changed = True




# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.frontends.qt.meta import *
import os, string

from wallaby.qt_combat import *
from ..baseWidget import *
from ..logics import *

class Container(BaseWidget, EnableLogic):
    __metaclass__ = QWallabyMeta

    room = Meta.property("string")
    sheets = Meta.property("string")

    def __init__(self, *args):
        BaseWidget.__init__(self, *args)
        EnableLogic.__init__(self)

        self._sheetPrototypes = {}
        self._lastSheets = None
        self.sheetCache = {}
        self._sheet = None

    def activate(self):
        if self._sheet is not None:
            self.addChildWidget(self._sheet)

            # also activate nested containers
            containers = self._sheet.findChildren(Container)
            for c in containers: c.activate()

    def isMultiPage(self):
        return False

    def getChildWidget(self, pos):
        return None

    def childCount(self):
        return 0

    def addChildWidget(self, w):
        pass

    def removeChildWidget(self, w):
        w.hide()
        w.setParent(None)
        w.deleteLater()

    def clear(self):
        while self.childCount() > 0:
            widget = self.getChildWidget(0)
            if widget is None: next

            widgets = widget.findChildren(BaseWidget)
            for w in widgets:
                w.deregister(remove=True)
                w.destroyOverlay()
                # BaseWidget.widgets.remove(w)

            if isinstance (widget, BaseWidget):
                widget.deregister(remove=True)
                widget.destroyOverlay()
                # BaseWidget.widgets.remove(w)
                # BaseWidget.widgets.remove(widget)

            self.removeChildWidget(widget)

    def loadUIFiles(self):
        # Do not reload widgets if not changed
        if self.sheets is None or self.sheets == self._lastSheets: return
        self._lastSheets = self.sheets

        self.clear()

        isheets = []

        self._sheetPrototypes = {}

        import wallaby.frontends as frontends
        from twisted.plugin import getCache

        for p in FX.packagePath(FX.appModule + ".isheet"): isheets.append(p)
        for frontend in getCache(frontends):
            for p in FX.packagePath("wallaby.frontends." + frontend + ".isheet"): isheets.append(p)

        import os
        for path in isheets:
            root = re.sub(r'[/\\]','.', path).replace('..', '')
            root = re.sub(r'^.*wallaby\.', 'wallaby.', root)
            files = os.listdir(path)
            for file in files:
                if '.py' in file and '.pyc' not in file and 'UI_' in file:
                    basename, ext = os.path.splitext(file)

                    if ext != '.py': continue

                    mod = FX.imp(root + '.' + basename)
                    if mod == None: continue

                    cls = basename[0] + string.lower(basename[1]) + basename[2] + string.upper(basename[3]) + basename[4:]
    
                    sheetName = unicode(basename.replace('UI_', ''))

                    sheetCache = None

                    if sheetName in self._sheetPrototypes: 
                        continue

                    if re.match(self.sheets, sheetName) is None: 
                        continue

                    if not self.isMultiPage():
                        p = self.parent()
                        while p != None and (not isinstance(p, Container) or not p.isMultiPage()):
                            p = p.parent()

                        if p is not None:
                            sheetCache = p.sheetCache

                            if sheetName in sheetCache:
                                self._sheet = sheetCache[sheetName]
                                return

                    if not cls in mod.__dict__:
                        for key, val in mod.__dict__.items():
                            if key.startswith("Ui_"):
                                cls = key
                                break

                    if not cls in mod.__dict__: continue

                    self._sheetPrototypes[sheetName] = mod.__dict__[cls]

                    sheet = QtGui.QWidget(self)
                    sheet.ui = self._sheetPrototypes[sheetName]()
                    sheet.ui.setupUi(sheet)
                    sheet.setObjectName(sheetName + "Sheet") 

                    if sheetCache is not None:
                        self._sheet = sheet
                        sheetCache[sheetName] = sheet
                        # also add the cached sheet to the current widget
                        # to allow nested widgets to also be loaded

                    self.addChildWidget(sheet)

    def deregister(self, remove):
        EnableLogic.register(self, remove)

    def register(self):
        EnableLogic.register(self)

        if self.sheets != None:
            self.loadUIFiles()



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/dominik/Documents/Projekte/wallaby/wallaby-frontend-qt/wallaby/frontends/qt/isheet/gbContainer.ui'
#
# Created: Thu Nov 29 16:59:25 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName(_fromUtf8("Form"))
        Form.resize(412, 46)
        self.horizontalLayout = QtGui.QHBoxLayout(Form)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.label = QtGui.QLabel(Form)
        self.label.setObjectName(_fromUtf8("label"))
        self.horizontalLayout.addWidget(self.label)
        self.LineEdit = LineEdit(Form)
        self.LineEdit.setObjectName(_fromUtf8("LineEdit"))
        self.horizontalLayout.addWidget(self.LineEdit)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(QtGui.QApplication.translate("Form", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Form", "Name-pattern for selecting embedded sheets", None, QtGui.QApplication.UnicodeUTF8))
        self.LineEdit.setProperty("template", QtGui.QApplication.translate("Form", "inspector-sheets", None, QtGui.QApplication.UnicodeUTF8))

from wallaby.frontends.qt.widgets.edits.lineEdit import LineEdit



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *
from ..baseWidget import *
from UI_prefSheet import *
from functools import partial
import json
from wallaby.common.draggable import *
from lockLayer import LockLayer

class PrefSheet(QtGui.QWidget, Ui_PrefSheet, BaseWidget, Draggable):
    __metaclass__ = QWallabyMeta

    room = Meta.property("string")
    path = Meta.property("string")

    lockLayerTopMargin = 18

    Up = None
    Down = None

    # def __del__(self):
    #    print "!!!DELETING!!!", self

    def overlayRect(self):
        r = BaseWidget.overlayRect(self)
        # r.moveTo(r.x(), r.height()-20)
        r.setHeight(min(r.height(), 20))
        return r

    def __init__(self, type, room, path, controller, idx, title, locked=True):
        QtGui.QWidget.__init__(self)
        BaseWidget.__init__(self, QtGui.QWidget)

        self.setupUi(self)

        self.room = room
        self._type = type
        self._idx = idx
        self._path = path
        self.path = path + "." + unicode(self._idx)
        self._controller = controller
        self.groupBox.setTitle(title)

        self.remove.clicked.connect(self.removePrefWidget)

        self.moveUp.clicked.connect(partial(self.movePrefWidget, True))
        self.moveDown.clicked.connect(partial(self.movePrefWidget, False))

        # self.content.setEnabled(False)
        self.edit.clicked.connect(self.editPrefWidget)

        self.setAcceptDrops(True)

        if PrefSheet.Up == None:
            PrefSheet.Up = QtGui.QPixmap(':icons/images/arrow_up.png')

        if PrefSheet.Down == None:
            PrefSheet.Down = QtGui.QPixmap(':icons/images/arrow_down.png')

        self._dragging = False
        self._direction = True
        self._ensureVisibleCall = None

        self.groupBox.layout().takeAt(0)
        self.toolbar.setParent(self)
        self.toolbar.move(300, -15)
        self.toolbar.setMinimumHeight(50)

        self._lockLayer = LockLayer()
        self._lockLayer.setParent(self)
        self._lockLayer.move(0,PrefSheet.lockLayerTopMargin)

        if locked:
            self.lock() 
        else:
            self.unlock()
        
        self.update()

    def unlock(self):
        self._lockLayer.hide()  
        self.content.setEnabled(True)
        self.edit.setChecked(True)

    def lock(self):
        self._lockLayer.show()  
        self.content.setEnabled(False)
        self.edit.setChecked(False)

    def resizeEvent(self, e):
        QtGui.QWidget.resizeEvent(self, e)
        if self.height() > PrefSheet.lockLayerTopMargin:
            self._lockLayer.setMinimumHeight(self.height()-PrefSheet.lockLayerTopMargin)
            self._lockLayer.setMinimumWidth(self.width())
            self._lockLayer.setMaximumHeight(self.height()-PrefSheet.lockLayerTopMargin)
            self._lockLayer.setMaximumWidth(self.width())

    def dragCustom(self, mimeData):
        mimeData.setData('wallaby/pref', unicode(self._idx))

    def dragEnterEvent(self, event):
        if event.mimeData().hasFormat("wallaby/pref"):
            idx = int(event.mimeData().data("wallaby/pref"))
            self._checkDragEvent(event)

            if idx == self._idx: return
            event.acceptProposedAction()

    def _checkDragEvent(self, event):
        if event.mimeData().hasFormat("wallaby/pref"):
            if self._ensureVisibleCall is None:
                from twisted.internet import reactor
                self._ensureVisibleCall = reactor.callLater(0.5, self._callEnsureVisible)

            idx = int(event.mimeData().data("wallaby/pref"))
            if idx == self._idx: 
                self._rejectDragEvent(event)
                return

            if event.pos().y() < self.height()/2:
                if idx == self._idx - 1: 
                    self._rejectDragEvent(event)
                    return

                self._direction = True
            else:
                if idx == self._idx + 1: 
                    self._rejectDragEvent(event)
                    return

                self._direction = False

            self._dragging = True
            event.acceptProposedAction()

            self.update()
            return 
        self._rejectDragEvent(event)

    def _rejectDragEvent(self, event):
        if self._dragging:
            self._dragging = False
            self.update()

    def _callEnsureVisible(self):
        self._ensureVisibleCall = None
        self._controller.scrollArea.ensureWidgetVisible(self, 50, 100)

    def dragMoveEvent(self, event):
        self._checkDragEvent(event)
    
    def dragLeaveEvent(self, event):
        self._dragging = False
        if self._ensureVisibleCall is not None:
            self._ensureVisibleCall.cancel()
            self._ensureVisibleCall = None

        self.update()

    def dropEvent(self, event):
        self._dragging = False

        if self._ensureVisibleCall is not None:
            self._ensureVisibleCall.cancel()
            self._ensureVisibleCall = None

        idx = int(event.mimeData().data("wallaby/pref"))
        if event.pos().y() < self.height()/2:
            self._controller.move(idx, self._idx)
        else:
            self._controller.move(idx, self._idx+1)

        event.acceptProposedAction()
        self.releaseMouse()
        self.setMouseTracking(False)
        self.update()

    def paintEvent(self, e):
        if self._dragging:
            p = QtGui.QPainter(self)
            if self._direction:
                p.drawPixmap(self.width()/2.0-PrefSheet.Up.width()/2.0,20, PrefSheet.Up)
            else:
                p.drawPixmap(self.width()/2.0-PrefSheet.Down.width()/2.0,self.height()-PrefSheet.Down.height(), PrefSheet.Down)

    def type(self):
        return self._type

    def dragText(self):
        if self._controller._pref == None: return None
        return json.dumps(self._controller._pref.document().get(self.path))

    def dragURL(self):
        return self._controller._pref.document().url()

    def editPrefWidget(self):
        if self.edit.isChecked():
            self.unlock()
        else:
            self.lock()

    def movePrefWidget(self, direction, state):
        self._controller.movePrefSheet(self._idx, direction)

    def decrementIndex(self):
        self.setIndex(self._idx - 1)

    def incrementIndex(self):
        self.setIndex(self._idx + 1)

    def setIndex(self, idx):
        self._idx = idx
        self.path = self._path + "." + unicode(self._idx)

        widgets = self.findChildren(BaseWidget)
        for w in widgets:
            w.deregister()
            w.register()

    def removePrefWidget(self):
        self._controller.remove(self._idx)

    def configure(self, config):
        pass




# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

import sys

from ..baseWidget import *
from ..logics import *

from wallaby.pf.peer.doubleViewer import *
from wallaby.pf.peer.doubleEditor import *

class DoubleSpinBox(QtGui.QDoubleSpinBox, BaseWidget, EnableLogic, ViewLogic, EditLogic, TriggeredPillowsLogic):
    __metaclass__ = QWallabyMeta

    throwWhileEditing = Meta.property("bool", extended=True, default=True)
    triggers = Meta.property("list", readOnly=True, default=["", "changed"])

    def __init__(self, *args):
        QtGui.QDoubleSpinBox.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QDoubleSpinBox, *args)
        EnableLogic.__init__(self)
        ViewLogic.__init__(self, DoubleViewer, self._setValue)
        EditLogic.__init__(self, DoubleEditor, self.value)
        TriggeredPillowsLogic.__init__(self)

        self._changed = True

        self._readOnly = self.isReadOnly()
        self.valueChanged.connect(self._spin)

    def _setEnabled(self, enabled):
        if self._readOnly: return

        if self.hideIfDisabled:
            self.setVisible(enabled)
        else:
            self.setReadOnly(not enabled)

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        ViewLogic.deregister(self, remove)
        EditLogic.deregister(self, remove)
        TriggeredPillowsLogic.deregister(self, remove)

    def register(self):
        EnableLogic.register(self)
        ViewLogic.register(self)
        EditLogic.register(self)
        TriggeredPillowsLogic.register(self)

    def _resolve(self, value, **ka):
        if value==None:
            self._setValue(0.0)
        else:
            self._setValue(value)

        if self._editor: self._editor._resolve(**ka)

    def _value(self):
        val = self.value()
        if self.decimals() == 0:
            return int(val)
        else:
            return val

    def _spin(self, val):
        self.trigger("changed")
        if self._editor and self._changed:
            self._editor._fieldChanged()

    def _setValue(self, value):
        self._changed = False
        if isinstance(value, basestring): value = float(value)
        self.setValue(value)
        self._changed = True




# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *
import sys

from ..baseWidget import *
from ..logics import *

from wallaby.pf.peer.viewer import *
from wallaby.pf.peer.listEditor import *

class CheckBox(QtGui.QCheckBox, BaseWidget, EnableLogic, ViewLogic, EditLogic, TriggeredPillowsLogic):
    __metaclass__ = QWallabyMeta

    value = Meta.property("string")
    triggers = Meta.property("list", default=["","clicked", "pressed", "released", "checked", "unchecked"])

    def __init__(self, *args):
        QtGui.QCheckBox.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QCheckBox, *args)
        EnableLogic.__init__(self)
        ViewLogic.__init__(self, Viewer, self.viewCallback)
        EditLogic.__init__(self, ListEditor, self.editCallback)
        TriggeredPillowsLogic.__init__(self)

        self.clicked.connect(self._clicked)

        from functools import partial
        self.clicked.connect(partial(self.trigger, "clicked", None))
        self.pressed.connect(partial(self.trigger, "pressed"))
        self.released.connect(partial(self.trigger, "released"))

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        ViewLogic.deregister(self, remove)
        EditLogic.deregister(self, remove)
        TriggeredPillowsLogic.deregister(self, remove)

    def _clicked(self, state):
        if state: self.trigger("checked") 
        else: self.trigger("unchecked")

        if self._editor: self._editor._fieldChanged()

    def register(self):
        EnableLogic.register(self)
        ViewLogic.register(self)
        EditLogic.register(self)
        TriggeredPillowsLogic.register(self)

    def _resolve(self, value, **ka):
        self.viewCallback(value)
        if self._editor: 
            self._editor._resolve(**ka)
            self._editor._fieldChanged()

    def viewCallback(self, value):
        exists = False
        if isinstance(value, list) or isinstance(value, tuple):
            if self.value in value: exists = True

        elif value in (True, 'true', 'True') and self.value in ('True', 'true'):
            exists = True
        elif value in (False, 'false', 'False') and self.value in ('False', 'false'):
            exists = True 

        if isinstance(exists, bool):
            self.setChecked(exists)
        else:
            self.setChecked(False)

    def editCallback(self):
        return self




# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

from setuptools import setup, find_packages
import platform, sys

setup(name='wallaby-frontend-qt',
      version='0.2.44',
      url='https://github.com/FreshXOpenSource/wallaby-frontend-qt',
      author='FreshX GbR',
      author_email='wallaby@freshx.de',
      license='BSD',
      description='Integration of Qt into wallaby.',
      long_description=open('README.md').read(),
      package_data={'': ['LICENSE', 'AUTHORS', 'README.md']},
      classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: MacOS X',
        'Environment :: Win32 (MS Windows)',
        'Environment :: X11 Applications',
        'Framework :: Twisted',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: BSD License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Application Frameworks'
      ],
      scripts=["wlby"],
      packages=find_packages('.'),
      include_package_data = True,
      install_requires=['wallaby-base','markdown'] # and PyQt or PySide ... currently from brew/yum/apt-get
  )



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

class Options:
    def __init__(self):
        self.server = "127.0.0.1"
        self.app = "$appname$"
        self.db = "$appname$"
        self.password = self.username = None
        self.fx = True
        self.debug = ""

import warnings
warnings.simplefilter('ignore')

import wallaby.apps.wallabyApp
wallaby.apps.wallabyApp.WallabyApp("$appname$", options=Options())



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

from dragLogic import *

class DragLabel(QtGui.QLabel, DragLogic):

    def __init__(self, *args):
        QtGui.QLabel.__init__(self, *args)
        DragLogic.__init__(self)

    def mousePressEvent(self, event):
        DragLogic.mousePressEvent(self, event)

    def mouseMoveEvent(self, event):
        DragLogic.mouseMoveEvent(self, event)



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/dominik/Documents/Projekte/wallaby/wallaby-frontend-qt/wallaby/frontends/qt/isheet/gbTriggeredPillows.ui'
#
# Created: Fri Dec 28 12:01:13 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_GbRoomAndPath(object):
    def setupUi(self, GbRoomAndPath):
        GbRoomAndPath.setObjectName(_fromUtf8("GbRoomAndPath"))
        GbRoomAndPath.resize(669, 437)
        GbRoomAndPath.setMinimumSize(QtCore.QSize(0, 437))
        GbRoomAndPath.setMaximumSize(QtCore.QSize(16777215, 504))
        self.verticalLayout = QtGui.QVBoxLayout(GbRoomAndPath)
        self.verticalLayout.setMargin(0)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.embeddedViewTable = EmbeddedViewTable(GbRoomAndPath)
        self.embeddedViewTable.setMinimumSize(QtCore.QSize(0, 150))
        self.embeddedViewTable.setObjectName(_fromUtf8("embeddedViewTable"))
        self.verticalLayout.addWidget(self.embeddedViewTable)
        self.widget = QtGui.QWidget(GbRoomAndPath)
        self.widget.setObjectName(_fromUtf8("widget"))
        self.horizontalLayout = QtGui.QHBoxLayout(self.widget)
        self.horizontalLayout.setMargin(0)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.actionButton_2 = ActionButton(self.widget)
        self.actionButton_2.setObjectName(_fromUtf8("actionButton_2"))
        self.horizontalLayout.addWidget(self.actionButton_2)
        self.actionButton = ActionButton(self.widget)
        self.actionButton.setObjectName(_fromUtf8("actionButton"))
        self.horizontalLayout.addWidget(self.actionButton)
        self.verticalLayout.addWidget(self.widget)
        self.widget_2 = QtGui.QWidget(GbRoomAndPath)
        self.widget_2.setObjectName(_fromUtf8("widget_2"))
        self.gridLayout = QtGui.QGridLayout(self.widget_2)
        self.gridLayout.setMargin(0)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.label_3 = QtGui.QLabel(self.widget_2)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.gridLayout.addWidget(self.label_3, 2, 0, 1, 1)
        self.ComboBox = ComboBox(self.widget_2)
        self.ComboBox.setObjectName(_fromUtf8("ComboBox"))
        self.gridLayout.addWidget(self.ComboBox, 0, 2, 1, 1)
        self.label = QtGui.QLabel(self.widget_2)
        self.label.setMaximumSize(QtCore.QSize(100, 16777215))
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.label_2 = QtGui.QLabel(self.widget_2)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.ComboBox_2 = ComboBox(self.widget_2)
        self.ComboBox_2.setEditable(True)
        self.ComboBox_2.setObjectName(_fromUtf8("ComboBox_2"))
        self.gridLayout.addWidget(self.ComboBox_2, 1, 2, 1, 1)
        self.checkBox = CheckBox(self.widget_2)
        self.checkBox.setObjectName(_fromUtf8("checkBox"))
        self.gridLayout.addWidget(self.checkBox, 2, 2, 1, 1)
        self.verticalLayout.addWidget(self.widget_2)
        self.plainTextEdit = PlainTextEdit(GbRoomAndPath)
        self.plainTextEdit.setObjectName(_fromUtf8("plainTextEdit"))
        self.verticalLayout.addWidget(self.plainTextEdit)

        self.retranslateUi(GbRoomAndPath)
        QtCore.QMetaObject.connectSlotsByName(GbRoomAndPath)

    def retranslateUi(self, GbRoomAndPath):
        GbRoomAndPath.setWindowTitle(QtGui.QApplication.translate("GbRoomAndPath", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.embeddedViewTable.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-table-triggeredpillows", None, QtGui.QApplication.UnicodeUTF8))
        self.actionButton_2.setText(QtGui.QApplication.translate("GbRoomAndPath", "add", None, QtGui.QApplication.UnicodeUTF8))
        self.actionButton_2.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-triggeredpillows-add", None, QtGui.QApplication.UnicodeUTF8))
        self.actionButton.setText(QtGui.QApplication.translate("GbRoomAndPath", "remove", None, QtGui.QApplication.UnicodeUTF8))
        self.actionButton.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-triggeredpillows-remove", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("GbRoomAndPath", "Feathers:", None, QtGui.QApplication.UnicodeUTF8))
        self.ComboBox.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-triggers", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("GbRoomAndPath", "Trigger", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("GbRoomAndPath", "Pillow", None, QtGui.QApplication.UnicodeUTF8))
        self.ComboBox_2.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-cb-pillow", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBox.setText(QtGui.QApplication.translate("GbRoomAndPath", "Is JSON", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBox.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-cb-isjson", None, QtGui.QApplication.UnicodeUTF8))
        self.plainTextEdit.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-pe-feathers", None, QtGui.QApplication.UnicodeUTF8))

from wallaby.frontends.qt.widgets.buttons.checkBox import CheckBox
from wallaby.frontends.qt.widgets.multi.embeddedViewTable import EmbeddedViewTable
from wallaby.frontends.qt.widgets.combo.comboBox import ComboBox
from wallaby.frontends.qt.widgets.buttons.actionButton import ActionButton
from wallaby.frontends.qt.widgets.edits.plainTextEdit import PlainTextEdit



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/dominik/Documents/Projekte/wallaby/wallaby-frontend-qt/wallaby/frontends/qt/isheet/gbQuery.ui'
#
# Created: Sun Dec 16 18:08:31 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_GbRoomAndPath(object):
    def setupUi(self, GbRoomAndPath):
        GbRoomAndPath.setObjectName(_fromUtf8("GbRoomAndPath"))
        GbRoomAndPath.resize(652, 445)
        self.verticalLayout = QtGui.QVBoxLayout(GbRoomAndPath)
        self.verticalLayout.setContentsMargins(0, 20, 0, 0)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.checkBox_3 = CheckBox(GbRoomAndPath)
        self.checkBox_3.setObjectName(_fromUtf8("checkBox_3"))
        self.verticalLayout.addWidget(self.checkBox_3)
        self.groupBox_2 = QtGui.QGroupBox(GbRoomAndPath)
        self.groupBox_2.setObjectName(_fromUtf8("groupBox_2"))
        self.gridLayout = QtGui.QGridLayout(self.groupBox_2)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.LineEdit = LineEdit(self.groupBox_2)
        self.LineEdit.setObjectName(_fromUtf8("LineEdit"))
        self.gridLayout.addWidget(self.LineEdit, 1, 1, 1, 1)
        self.label = QtGui.QLabel(self.groupBox_2)
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout.addWidget(self.label, 1, 0, 1, 1)
        self.LineEdit_2 = LineEdit(self.groupBox_2)
        self.LineEdit_2.setObjectName(_fromUtf8("LineEdit_2"))
        self.gridLayout.addWidget(self.LineEdit_2, 2, 1, 1, 1)
        self.label_3 = QtGui.QLabel(self.groupBox_2)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.gridLayout.addWidget(self.label_3, 3, 0, 1, 1)
        self.label_2 = QtGui.QLabel(self.groupBox_2)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1)
        self.LineEdit_3 = LineEdit(self.groupBox_2)
        self.LineEdit_3.setObjectName(_fromUtf8("LineEdit_3"))
        self.gridLayout.addWidget(self.LineEdit_3, 3, 1, 1, 1)
        self.checkBox_2 = CheckBox(self.groupBox_2)
        self.checkBox_2.setObjectName(_fromUtf8("checkBox_2"))
        self.gridLayout.addWidget(self.checkBox_2, 0, 0, 1, 2)
        self.verticalLayout.addWidget(self.groupBox_2)
        self.groupBox = QtGui.QGroupBox(GbRoomAndPath)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.embeddedViewTable = EmbeddedViewTable(self.groupBox)
        self.embeddedViewTable.setMinimumSize(QtCore.QSize(0, 150))
        self.embeddedViewTable.setObjectName(_fromUtf8("embeddedViewTable"))
        self.verticalLayout_2.addWidget(self.embeddedViewTable)
        self.widget = QtGui.QWidget(self.groupBox)
        self.widget.setObjectName(_fromUtf8("widget"))
        self.horizontalLayout = QtGui.QHBoxLayout(self.widget)
        self.horizontalLayout.setMargin(0)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.actionButton_2 = ActionButton(self.widget)
        self.actionButton_2.setObjectName(_fromUtf8("actionButton_2"))
        self.horizontalLayout.addWidget(self.actionButton_2)
        self.actionButton = ActionButton(self.widget)
        self.actionButton.setObjectName(_fromUtf8("actionButton"))
        self.horizontalLayout.addWidget(self.actionButton)
        self.verticalLayout_2.addWidget(self.widget)
        self.verticalLayout.addWidget(self.groupBox)

        self.retranslateUi(GbRoomAndPath)
        QtCore.QMetaObject.connectSlotsByName(GbRoomAndPath)

    def retranslateUi(self, GbRoomAndPath):
        GbRoomAndPath.setWindowTitle(QtGui.QApplication.translate("GbRoomAndPath", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBox_3.setText(QtGui.QApplication.translate("GbRoomAndPath", "Fill combobox with values from view", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBox_3.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-isview", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setTitle(QtGui.QApplication.translate("GbRoomAndPath", "General Settings", None, QtGui.QApplication.UnicodeUTF8))
        self.LineEdit.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-view", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("GbRoomAndPath", "View", None, QtGui.QApplication.UnicodeUTF8))
        self.LineEdit_2.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-viewpath", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("GbRoomAndPath", "Path of displayed value", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("GbRoomAndPath", "Path of raw value", None, QtGui.QApplication.UnicodeUTF8))
        self.LineEdit_3.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-viewdisplaypath", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBox_2.setText(QtGui.QApplication.translate("GbRoomAndPath", "Use the view\'s values as data (data must not be loaded by document-id)", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBox_2.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-dataview", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("GbRoomAndPath", "View arguments", None, QtGui.QApplication.UnicodeUTF8))
        self.embeddedViewTable.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-table-viewargs", None, QtGui.QApplication.UnicodeUTF8))
        self.actionButton_2.setText(QtGui.QApplication.translate("GbRoomAndPath", "add", None, QtGui.QApplication.UnicodeUTF8))
        self.actionButton_2.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-viewargs-add", None, QtGui.QApplication.UnicodeUTF8))
        self.actionButton.setText(QtGui.QApplication.translate("GbRoomAndPath", "remove", None, QtGui.QApplication.UnicodeUTF8))
        self.actionButton.setProperty("template", QtGui.QApplication.translate("GbRoomAndPath", "inspector-viewargs-remove", None, QtGui.QApplication.UnicodeUTF8))

from wallaby.frontends.qt.widgets.buttons.checkBox import CheckBox
from wallaby.frontends.qt.widgets.multi.embeddedViewTable import EmbeddedViewTable
from wallaby.frontends.qt.widgets.edits.lineEdit import LineEdit
from wallaby.frontends.qt.widgets.buttons.actionButton import ActionButton



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *
import sys

from ..baseWidget import *

from wallaby.pf.peer.viewer import *
from wallaby.pf.peer.editor import *
from ..logics import *

class TextPopupButton(QtGui.QPushButton, BaseWidget, EnableLogic, ViewLogic, EditLogic):
    __metaclass__ = QWallabyMeta

    title = Meta.property("string")

    def __init__(self, *args):
        QtGui.QPushButton.__init__(self, *args)
        BaseWidget.__init__(self, QtGui.QPushButton, *args)
        EnableLogic.__init__(self)

        self._changed = True

        self._plainTextEdit = QtGui.QPlainTextEdit()
        self._plainTextEdit.textChanged.connect(self._textChanged)

        self._dialog = QtGui.QDialog(self)
        self._dialog.setWindowFlags(self._dialog.windowFlags() | QtCore.Qt.Tool)

        layout = QtGui.QHBoxLayout()
        self._dialog.setLayout(layout)
        self._dialog.hide()

        layout.addWidget(self._plainTextEdit)

        ViewLogic.__init__(self, Viewer, self._setValue)
        EditLogic.__init__(self, Editor, self._plainTextEdit.toPlainText)

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        ViewLogic.deregister(self, remove)
        EditLogic.deregister(self, remove)

    def register(self):
        EnableLogic.register(self)
        ViewLogic.register(self)
        EditLogic.register(self)

        if self.title != None:
            self._dialog.setWindowTitle(self.title)

        self.clicked.connect(self._buttonClicked)

    def _setEnabled(self, enabled):
        if self.hideIfDisabled:
            self.setVisible(enabled)
        else:
            self._plainTextEdit.setReadOnly(not enabled)

    def _buttonClicked(self, state):
        if self._dialog.isHidden():
            self._dialog.show()
            self._dialog.raise_()
        else:
            self._dialog.hide()

    def _resolve(self, value, **ka):
        self._setValue(value)
        if self._editor: self._editor._resolve(**ka)

    def _setValue(self, val):
        if self._plainTextEdit.toPlainText() != val:
            self._changed = False
            self._plainTextEdit.setPlainText(val)
            self._changed = True

    def _textChanged(self):
        if self._editor and self._changed:
            self._editor._fieldChanged()

    def _docCB(self, document):
        print "doc", document
        print self.objectName()
        print "obj name", document.get(self.objectName())




# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

from wallaby.common.sets import *
from ..baseWidget import *
from ..logics import *
from wallaby.pf.peer.actionButtonPeer import *
from wallaby.pf.peer.multiViewer import *
from imageDelegate import *

class MultiViewTable(QtGui.QTableView, BaseWidget, EnableLogic, ContextMenuLogic, TriggeredPillowsLogic):
    __metaclass__ = QWallabyMeta

    room       = Meta.property("string")
    # Delete if all apps are converted
    dbColumns  = Meta.property("list")
    tab        = Meta.property("string")

    sortable   = Meta.property("list")
    sizeHints  = Meta.property("dict")

    orderPath = Meta.property("string")

    queryDocID    = Meta.property("string")

    view           = Meta.property("string")
    viewArguments  = Meta.property("dict")
    viewIdentifier = Meta.property("string")
    dataView       = Meta.property("bool")

    idVisible = Meta.property("bool")

    doubleClickPillow   = Meta.property("string")
    doubleClickFeathers = Meta.property("string")
    doubleClickTab      = Meta.property("string")

    keyCodePillows      = Meta.property("list")

    autoResizeCells = Meta.property("bool")
    minRowHeight = Meta.property("int")
    minColumnWidth = Meta.property("int")

    multiSelect = Meta.property("bool")

    triggers = Meta.property("list", readOnly=True, default=["", "clicked", "double-clicked", "key"])

    def getMenuEntries(self): return ContextMenuLogic.getMenuEntries(self)

    def overlayLabel(self):
        rooms = OrderedSet() 
        rooms.add(self.room)

        if self.room == None:
            return [rooms]

        if self.viewIdentifier != self.room and self.viewIdentifier != None:
            rooms.add(self.viewIdentifier.upper())

        rooms |= House.pillowRooms(self.room, MultiViewer.Out.Select, None)

        if self.doubleClickPillow != None:
            rooms |= House.pillowRooms(self.room, self.doubleClickPillow, self.doubleClickFeathers)

        for room, pillow, feathers in self._keyCodePillows.values():
            rooms |= House.pillowRooms(room, pillow, feathers)
            
        if self.view != None:
            return (rooms, self.view)
        else:
            return (rooms, "external query")

    def __init__(self, parent = None):
        QtGui.QTableView.__init__(self, parent)
        BaseWidget.__init__(self, QtGui.QTableView)
        EnableLogic.__init__(self)
        TriggeredPillowsLogic.__init__(self)

        self.setSortingEnabled(False)
        self._sortable = {}
        self._sortField = {}
        self._columns = []
        self._labels = []
        self._autoColumns = []
        self._keyCodePillows = {}

        self._ignoreClick = False

        self.horizontalHeader().setStretchLastSection(True)

        self._popup = None

        self._actionPeer = None
        self._model = None

        self.horizontalHeader().setClickable(True)
        self.horizontalHeader().sectionClicked.connect(self._columnClicked)

        self.setAlternatingRowColors(True)
        self.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)

        self.verticalHeader().setClickable(True)
        self.verticalHeader().sectionClicked.connect(self._itemClicked)

        self.verticalHeader().sectionMoved.connect(self.reorderRows)

        self.clicked.connect(self._itemClicked)
        self.doubleClicked.connect(self._itemDoubleClicked)

        self._imageDelegate = ImageDelegate()
        self._progressDelegate = ProgressDelegate()

#         self.setDragDropMode(QtGui.QAbstractItemView.InternalMove)
#         self.viewport().setAcceptDrops(True)
#         self.setDropIndicatorShown(False)

    def reorderRows(self, row, oldIndex, newIndex):
        if self.model() != None:
            self.model().reorderRows(row, oldIndex, newIndex)

    def _columnClicked(self, col):
        if col-1 >= len(self._columns) or len(self._sortable) == 0:
            return

        column = self._columns[col-1]
        header = self.horizontalHeader()

        if column not in self._sortable:
            header.setSortIndicator(-1, 0)
            return False
        order = 0

        order = self._sortable[self._columns[col-1]]
        if header.sortIndicatorSection() == col:
            order = (order + 1) % 2
        else:
            order = 0

        header.setSortIndicator(col, order)

        self._sortable[column] = order
        self.model().sort(self._sortField[column], order)

    def _itemClicked(self, index):
        if self._ignoreClick:
            self._ignoreClick = False
            # return

        self.resetSelection()
        self.trigger("clicked")

    def keyPressEvent(self, event):
        accept = False

        import wallaby.FXUI as FXUI 
        if self.trigger("key", FXUI.key_event_to_name(event)):
            accept = True
 
        if event.modifiers() & QtCore.Qt.ControlModifier and event.key() in self._keyCodePillows:
            room, pillow, feathers = self._keyCodePillows[event.key()]
            House.get(room).throw(pillow, feathers)
            accept = True

        if accept:
            event.accept()
            QtGui.QTableView.keyPressEvent(self, event)        
        else:
            event.ignore()
            QtGui.QTableView.keyPressEvent(self, event)        
            self.resetSelection()

    def selectionChanged(self, selected, deselected):
        QtGui.QTableView.selectionChanged(self, selected, deselected)

        if self.multiSelect:
            sel = self.selectionModel().selectedIndexes()
            selection = set() 
            for s in sel:
                selection.add(s.row())

            self.model().doMultiSelect(selection)

    def initialData(self):
        if not self.autoResizeCells: return
        print "Initial"

        self.horizontalHeader().resizeSections(QtGui.QHeaderView.ResizeToContents)
        self.verticalHeader().resizeSections(QtGui.QHeaderView.ResizeToContents)
        self.horizontalHeader().setStretchLastSection(True)

        # for c in self._autoColumns:
        #    self.horizontalHeader().setResizeMode(c, QtGui.QHeaderView.ResizeToContents)

        if len(self._autoColumns) > 0:
            self.verticalHeader().setResizeMode(QtGui.QHeaderView.ResizeToContents)

    def clearSelection(self):
        self.selectionModel().clear()
        self.model().doSelect(None, None)

    def resetSelection(self):
        # FIXME
        # if self.autoResizeCells: self.initialData()
        sel = self.selectionModel().selectedIndexes()
        if len(sel) == 0:
            self.model().doSelect(None, None)
            # Ignore null selection
            # pass
        else:
            self.model().doSelect(sel[0].row(), self.tab)

    def _itemDoubleClicked(self, index):
        # FIXME
        self._ignoreClick = True

        self.trigger("double-clicked")

        if self._actionPeer:
            self._actionPeer.buttonClicked()

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        TriggeredPillowsLogic.deregister(self, remove)

        FXUI.mainWindow.settings().setValue(self._template + "_headerState", self.horizontalHeader().saveState())

        if self._model: self._model.destroyPeers(remove)
        if self._actionPeer: self._actionPeer.destroy(remove)

        self._actionPeer = None

    def register(self):
        EnableLogic.register(self)
        TriggeredPillowsLogic.register(self)

        self._sortable = {}
        self._sortField = {"__default__": "__default__"}

        sortable = []


        converted = False
        for c in self.sortable:
            if c is None: 
                sortable.append(None)
                continue
                
            if isinstance(c, (unicode, str)):
                if ':' in c:
                    c, field = c.split(':')
                else:
                    field = c

                sortable.append({'sortPath': field, 'path': c})
                converted = True
            else:
                sortable.append(c)

        if converted: self.sortable = sortable

        for c in self.sortable:
            if c is None: continue

            field = c.get('sortPath', None)
            c = c.get('path', None)
               
            self._sortable[c] = 0
            self._sortField[c] = field

        if len(self._sortable) > 0:
            self.horizontalHeader().setSortIndicatorShown(True)
        else:
            self.horizontalHeader().setSortIndicatorShown(False)

        if self.multiSelect:
            self.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        else:
            self.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)

        if isinstance(self.dbColumns, list) and len(self.dbColumns) > 0 and isinstance(self.dbColumns[0], (str, unicode)):
            columns, labels = FX.splitList(self.dbColumns, ':')
            types, args, columns = FX.extractType(columns, json=False)

            dbColumns = []

            # convert old format
            for i in range(len(columns)):
                dbColumns.append({
                    'path': columns[i],
                    'label': labels[i],
                    'type': types[i],
                    'type-args': args[i]
                })

            self.dbColumns = dbColumns

        self._columns = []
        self._labels = []
        self._types = []
        for cfg in self.dbColumns:
            if cfg is None: continue
            self._columns.append(cfg.get('path', None))
            self._labels.append(cfg.get('label', None))
            self._types.append(FX.convertType(cfg.get('type', None)))

        if self.orderPath != None:
            self.verticalHeader().setMovable(True) 

        if self.keyCodePillows != None:
            for kca in self.keyCodePillows:
                kca = unicode(kca)
                lst = kca.split(':')
                code = lst.pop(0)
                if len(lst) >= 2:
                    room = lst.pop(0)
                else:
                    room = self.room

                pillow = lst.pop(0)

                feathers = None
                if ',' in pillow:
                    feathers = pillow.split(',')
                    pillow = feathers.pop(0)

                code = eval("QtCore.Qt." + code)

                self._keyCodePillows[code] = (room, pillow, feathers)


        self._autoColumns = []
        col = 0
        for t in self._types:
            hasDelegate = False
            if isinstance(t, dict):
                for k, v in t.items():
                    if not hasDelegate and len(v) > 0 and v[0] == ':':
                        self.setItemDelegateForColumn(col+1, self._imageDelegate)
                        self._autoColumns.append(col+1)
                        hasDelegate = True
            elif t == "progress":
                self.setItemDelegateForColumn(col+1, self._progressDelegate)
            elif t == "image":
                self.setItemDelegateForColumn(col+1, self._imageDelegate)
                self._autoColumns.append(col+1)
                hasDelegate = True

            col = col + 1

        if self.doubleClickPillow != None:
            self._actionPeer = ActionButtonPeer(self.room, self.doubleClickPillow, self.doubleClickFeathers, tab=self.doubleClickTab)

        from wallaby.frontends.qt.models.multiViewTableModel import MultiViewTableModel
        self._model = MultiViewTableModel(self, self.dbColumns, room=self.room, view=self.view, viewIdentifier=self.viewIdentifier, viewArguments=self.viewArguments, dataView=self.dataView, orderPath=self.orderPath, minRowHeight=self.minRowHeight, minColumnWidth=self.minColumnWidth, queryDocID=self.queryDocID, sizeHints=self.sizeHints)
        self.setModel(self._model)

        if self.idVisible:
            self.setColumnHidden(0, False)
        else:
            self.setColumnHidden(0, True)





# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/dominik/Documents/Projekte/wallaby/wallaby-frontend-qt/wallaby/frontends/qt/isheet/iDateEdit.ui'
#
# Created: Sun Dec 16 18:13:19 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_ILineEdit(object):
    def setupUi(self, ILineEdit):
        ILineEdit.setObjectName(_fromUtf8("ILineEdit"))
        ILineEdit.resize(406, 236)
        self.verticalLayout = QtGui.QVBoxLayout(ILineEdit)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.groupBox = GroupBox(ILineEdit)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.verticalLayout.addWidget(self.groupBox)
        self.groupBox_4 = QtGui.QGroupBox(ILineEdit)
        self.groupBox_4.setObjectName(_fromUtf8("groupBox_4"))
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.groupBox_4)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.checkBox = CheckBox(self.groupBox_4)
        self.checkBox.setObjectName(_fromUtf8("checkBox"))
        self.verticalLayout_2.addWidget(self.checkBox)
        self.verticalLayout.addWidget(self.groupBox_4)
        self.groupBox_3 = GroupBox(ILineEdit)
        self.groupBox_3.setObjectName(_fromUtf8("groupBox_3"))
        self.verticalLayout.addWidget(self.groupBox_3)
        self.groupBox_2 = GroupBox(ILineEdit)
        self.groupBox_2.setObjectName(_fromUtf8("groupBox_2"))
        self.verticalLayout.addWidget(self.groupBox_2)
        spacerItem = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)

        self.retranslateUi(ILineEdit)
        QtCore.QMetaObject.connectSlotsByName(ILineEdit)

    def retranslateUi(self, ILineEdit):
        ILineEdit.setWindowTitle(QtGui.QApplication.translate("ILineEdit", "LineEdit preferences", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("ILineEdit", "Basic settings", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setProperty("template", QtGui.QApplication.translate("ILineEdit", "inspector-groupbox-roomandpath", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_4.setTitle(QtGui.QApplication.translate("ILineEdit", "Date settings", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBox.setText(QtGui.QApplication.translate("ILineEdit", "Default to current date", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBox.setProperty("template", QtGui.QApplication.translate("ILineEdit", "inspector-defaulttonow", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_3.setTitle(QtGui.QApplication.translate("ILineEdit", "Triggered pillows", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_3.setProperty("template", QtGui.QApplication.translate("ILineEdit", "inspector-groupbox-triggeredpillows", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setTitle(QtGui.QApplication.translate("ILineEdit", "Enable logic", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setProperty("template", QtGui.QApplication.translate("ILineEdit", "inspector-groupbox-enablelogic", None, QtGui.QApplication.UnicodeUTF8))

from wallaby.frontends.qt.widgets.buttons.checkBox import CheckBox
from wallaby.frontends.qt.widgets.containers.groupBox import GroupBox



#!/usr/bin/env python
# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.


import wallaby.FX as FX
import wallaby.FXUI as FXUI

from wallaby.qt_combat import *

from wallaby.pf.peer import *
from wallaby.frontends.qt.widgets.baseWidget import *
import wallaby.backends.couchdb as couch
from wallaby.pf.peer.debugger import *
from wallaby.frontends.qt.widgets.wallabyOverlay import *
from wallaby.apps.baseApp import *

from twisted.internet import defer

class BaseWindow(BaseApp, QtGui.QMainWindow):
    def __init__(self, vendorName, appName, options, quitCB, parent=None, dbName=None, embedded=False, *args, **ka):
        BaseApp.__init__(self, quitCB, dbName, embedded=embedded, appName=appName)
        QtGui.QMainWindow.__init__(self, parent)

        self._options = options

        self._embedded = embedded

        # self.setAttribute(QtCore.Qt.WA_MacBrushedMetal)

        if not embedded:
            FX.mainWindow = self
            self.setEnabled(False)

        self._vendorName = vendorName
        self._appName = appName
        self._showOverlay = True

        self._splash = None

        self._settings = QtCore.QSettings(vendorName, appName)

    def settings(self):
        return self._settings

    def configure(self):
        geo = self._settings.value("geometry")
        winState = self._settings.value("windowState")
        if geo is not None: self.restoreGeometry(geo)
        if winState is not None: self.restoreState(winState)

    def setSplash(self, splash):
        self._splash = splash

    def overlayMode(self):
        return not self._showOverlay

    def options(self):
        return self._options

    def hideConfig(self):
        if FXUI.configEditor and FXUI.configEditor.isVisible():
            FXUI.configEditor.hide()

    def openConfig(self, widget):
        if FXUI.configEditor == None and (self._options is None or self._options.app != 'inspector'):
            from wallaby.apps.inspector.mainWindow import MainWindow
            FXUI.configEditor = MainWindow(self.hideConfig, self._options, embedded=True)

            if self._options is not None:
                FXUI.configEditor.authenticated(self._options.username, self._options.password, self._options)
            else:
                FXUI.configEditor.authenticated()

        if FXUI.configEditor != None and not FXUI.configEditor.isVisible(): FXUI.configEditor.show()

        from wallaby.pf.peer.documentChanger import DocumentChanger
        from wallaby.pf.peer.tab import Tab
        House.get("__CONFIG__").throw(DocumentChanger.In.Select, ("widgets", widget._template) )
        House.get("__CONFIG__").throw(Tab.In.Select, "main.0")
        if FXUI.configEditor != None: FXUI.configEditor.raise_()
        else: 
            if self._options is None or self._options.app != 'inspector':
                self.toggleOverlay()

    def setDebuggedRooms(self, rooms):
        self._debuggedRooms = rooms

    def keyPressEvent(self, e):

        if e.modifiers() & QtCore.Qt.ControlModifier and e.modifiers() & QtCore.Qt.AltModifier and e.key() == QtCore.Qt.Key_S:
            tl = FXUI.app.topLevelAt(QtGui.QCursor.pos())
            if tl != FXUI.mainWindow: return

            widget = FXUI.app.widgetAt(QtGui.QCursor.pos())

            if widget == None: return
            while not isinstance(widget, BaseWidget): 
                if widget.parent() == None: return
                widget = widget.parent()

            self.openConfig(widget)

        elif e.modifiers() & QtCore.Qt.ControlModifier and e.modifiers() & QtCore.Qt.AltModifier and e.key() == QtCore.Qt.Key_O:
            FXUI.mainWindow.toggleOverlay()
        else:
            QtGui.QMainWindow.keyPressEvent(self, e)

    def toggleOverlay(self):
        widgets = self.findChildren(BaseWidget)
        for w in widgets:
            if self._showOverlay:
                w.showOverlay()
            else: 
                w.hideOverlay()

        WallabyOverlay.resetColors()

        self._showOverlay = not self._showOverlay

    def resizeEvent(self, e):
        QtGui.QMainWindow.resizeEvent(self, e)
        if self.overlayMode():
            widgets = self.findChildren(BaseWidget)
            for w in widgets:
                w.resizeOverlay() 

    def reloadOverlays(self):
        if self.overlayMode():
            widgets = self.findChildren(BaseWidget)
            for w in widgets:
                w.hideOverlay()

            WallabyOverlay.resetColors()

            for w in widgets:
                w.showOverlay()

    def isRunning(self):
        self.setEnabled(True)

        from twisted.internet import reactor
        if self._splash != None: reactor.callLater(0.3, self._splash.finish, self)

    def closeEvent(self, e):
        settings = QtCore.QSettings(self._vendorName, self._appName)
        settings.setValue("geometry", self.saveGeometry())
        settings.setValue("windowState", self.saveState())
        settings.sync()
        self.quit()
        e.ignore()



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

from setuptools import setup, find_packages
 
setup(name='$appname$',
      version='0.1',
      url='<url>',
      author='<author>',
      author_email='<mail>',
      packages=find_packages('.'),
      install_requires=['wallaby-app-inspector'],
      include_package_data = True
  )



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX

from wallaby.qt_combat import *

from ..baseWidget import *
from ..logics import *
from wallaby.pf.peer.editor import *

from imageDelegate import *
from comboBoxDelegate import *

class EmbeddedViewTable(QtGui.QTableView, BaseWidget, EnableLogic, EditLogic, TriggeredPillowsLogic):
    __metaclass__ = QWallabyMeta

    room         = Meta.property("string")
    path         = Meta.property("string")
    reverseOrder = Meta.property("bool")
    identifier   = Meta.property("string")
    isList       = Meta.property("bool", default=True)
    dbColumns    = Meta.property("list")
    wrapInList   = Meta.property("bool")
    sizeHints  = Meta.property("dict")

    notSelectableIfDisabled = Meta.property("bool", default=False)

    doubleClickPillow   = Meta.property("string", extended=True)
    doubleClickFeathers = Meta.property("string", extended=True)
    doubleClickTab      = Meta.property("string", extended=True)

    autoResizeCells = Meta.property("bool", extended=True)
    minRowHeight = Meta.property("int", extended=True)
    minColumnWidth = Meta.property("int", extended=True)

    editOnInsert = Meta.property("bool")

    triggers = Meta.property("list", readOnly=True, default=["", "clicked", "double-clicked", "key"])

    def __init__(self, parent = None):
        QtGui.QTableView.__init__(self, parent)
        BaseWidget.__init__(self, QtGui.QTableView)
        EnableLogic.__init__(self)
        EditLogic.__init__(self, Editor, self.getData)
        TriggeredPillowsLogic.__init__(self)

        self.setSortingEnabled(False)

        self.horizontalHeader().setStretchLastSection(True)

        self._popup = None
        self._model = None
        self._actionPeer = None
        self._ignoreClick = False

        self._proxyModel = None
        self._selectedRow = None

        self._isEnabled = False

        self.setAlternatingRowColors(True)
        self.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)

        self.verticalHeader().sectionMoved.connect(self.reorderRows)
        self.verticalHeader().setMovable(False) 

        self.verticalHeader().sectionClicked.connect(self._itemClicked)

        self.clicked.connect(self._itemClicked)
        self.doubleClicked.connect(self._itemDoubleClicked)

        self._autoColumns = []

        self._imageDelegate = ImageDelegate()
        self._progressDelegate = ProgressDelegate()
        self._comboDelegates = []

    def editCell(self, row, col):
        idx = self._model.createIndex(row, col)
        if self._proxyModel is not None:
            idx = self._proxyModel.mapFromSource(idx)

        print "Set index", QtGui.QTableView.setCurrentIndex(self, idx)
        print "EDIT", QtGui.QTableView.edit(self, idx, QtGui.QTableView.AllEditTriggers, None)  

    def selectRow(self, row):
        if self._proxyModel is not None:
            idx = self._model.createIndex(row, 0)
            idx = self._proxyModel.mapFromSource(idx)
            row = idx.row()   

        QtGui.QTableView.selectRow(self, row)   

    def newData(self):
        if not self.autoResizeCells: return

        self.horizontalHeader().resizeSections(QtGui.QHeaderView.ResizeToContents)
        self.verticalHeader().resizeSections(QtGui.QHeaderView.ResizeToContents)
        self.horizontalHeader().setStretchLastSection(True)

        # for c in self._autoColumns:
        #    self.horizontalHeader().setResizeMode(c, QtGui.QHeaderView.ResizeToContents)

        if len(self._autoColumns) > 0:
            self.verticalHeader().setResizeMode(QtGui.QHeaderView.ResizeToContents)

    def _resolve(self, value, **ka):
        self._model.resolve(data=value, **ka)

    def getData(self):
        self._model.getData()

    def deregister(self, remove=False):
        EnableLogic.deregister(self, remove)
        EditLogic.deregister(self, remove)
        TriggeredPillowsLogic.deregister(self, remove)

        if self._model:
            FXUI.mainWindow.settings().setValue(self._template + "_headerState", self.horizontalHeader().saveState())

        if self._model: self._model.destroyPeers(remove)
        if self._actionPeer: self._actionPeer.destroy(remove)
        self._actionPeer = None

    def reorderRows(self, row, oldIndex, newIndex):
        if not self._isEnabled: return

        if self._model != None:
            self._model.reorderRows(row, oldIndex, newIndex)

    def register(self):
        EnableLogic.register(self)
        EditLogic.register(self)
        TriggeredPillowsLogic.register(self)

        if isinstance(self.dbColumns, list) and len(self.dbColumns) > 0 and isinstance(self.dbColumns[0], (str, unicode)):
            columns, labels = FX.splitList(self.dbColumns, ':')
            types, args, columns = FX.extractType(columns, json=False)

            dbColumns = []

            # convert old format
            for i in range(len(columns)):
                dbColumns.append({
                    'path': columns[i],
                    'label': labels[i],
                    'type': types[i],
                    'type-args': args[i]
                })

            self.dbColumns = dbColumns
 

        self._columns = []
        self._labels = []
        self._types = []
        for cfg in self.dbColumns:
            if cfg is None: continue

            self._columns.append(cfg.get('path', None))
            self._labels.append(cfg.get('label', None))
            self._types.append(FX.convertType(cfg.get('type', None)))

        self._autoColumns = []
        self._comboDelegate = []
        col = 0
        for t in self._types:
            hasDelegate = False
            if t is None: 
                col = col + 1
                continue
            if isinstance(t, dict):
                for k, v in t.items():
                    if not hasDelegate and len(v) > 0 and v[0] == ':':
                        self.setItemDelegateForColumn(col, self._imageDelegate)
                        self._autoColumns.append(col)
                        hasDelegate = True
            elif "image" in t:
                self.setItemDelegateForColumn(col, self._imageDelegate)
                self._autoColumns.append(col)
                hasDelegate = True
            elif t == "progress":
                self.setItemDelegateForColumn(col, self._progressDelegate)
            elif t == "comboedit":
                print "set combo delegate"
                self._comboDelegate.append(ComboBoxDelegate(self, col))
                self.setItemDelegateForColumn(col, self._comboDelegate[-1])

            col = col + 1

        from wallaby.frontends.qt.models.embeddedViewTableModel import EmbeddedViewTableModel
        self._model = EmbeddedViewTableModel(self, self.room, self.path, self.dbColumns, conflictCB=self._conflict, isList=self.isList, identifier=self.identifier, reverseOrder=self.reverseOrder, minRowHeight=self.minRowHeight, minColumnWidth=self.minColumnWidth, wrapInList=self.wrapInList, sizeHints=self.sizeHints, editOnInsert=self.editOnInsert)

        if FXUI.mainWindow.options() != None and FXUI.mainWindow.options().app != "inspector":
            self._proxyModel = QtGui.QSortFilterProxyModel(self)
            self._proxyModel.setSourceModel(self._model)
            self._proxyModel.setFilterRegExp(QtCore.QRegExp("^(?!inspector-|itest-)", QtCore.Qt.CaseInsensitive, QtCore.QRegExp.RegExp2))
            # proxyModel.setFilterRegExp(QtCore.QRegExp("^inspector", QtCore.Qt.CaseInsensitive, QtCore.QRegExp.RegExp2))

            # proxyModel.setFilterKeyColumn(0) 
            self.setModel(self._proxyModel)
        else:
            self.setModel(self._model)

        if self.doubleClickPillow != None:
            self._actionPeer = ActionButtonPeer(self.room, self.doubleClickPillow, self.doubleClickFeathers, tab=self.doubleClickTab, translate=True)

        QtCore.QTimer.singleShot(0, self.restoreSettings) 

    def restoreSettings(self):
        headerState = FXUI.mainWindow.settings().value(self._template + "_headerState")
        # if headerState: self.horizontalHeader().restoreState(headerState)

    def _itemDoubleClicked(self, index):
        # FIXME
        self._ignoreClick = True

        self.trigger("double-clicked")
        if self._actionPeer:
            self._actionPeer.buttonClicked(index.row())

    def _itemClicked(self, index):
        if self._ignoreClick:
            self._ignoreClick = False
            # return

        self.resetSelection()
        self.trigger("clicked")

    def setEnabled(self, enabled):
        self._isEnabled = enabled

        self.verticalHeader().setMovable(enabled) 

        if self.notSelectableIfDisabled:
            QtGui.QTableView.setEnabled(self, enabled)
            return

        if not enabled:
            self.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        else:
            self.setEditTriggers(QtGui.QAbstractItemView.DoubleClicked)

    def keyPressEvent(self, event):
        import wallaby.FXUI as FXUI 
        if self.trigger("key", FXUI.key_event_to_name(event)):
            event.accept()
        else:
            event.ignore()
            QtGui.QTableView.keyPressEvent(self, event)        

            if self.state() != QtGui.QAbstractItemView.EditingState:
                self.resetSelection()

    # def selectionChanged(self, selected, deselected):
    #     QtGui.QTableView.selectionChanged(self, selected, deselected)
    #     sel = self.selectionModel().selectedIndexes()
    #     if len(sel) == 0:
    #         self._model.doSelect(None)
    #     else:
    #         self._model.doSelect(sel[0].row())

    def resetSelection(self):
        sel = self.selectionModel().selectedIndexes()
        if len(sel) == 0:
            self._model.doSelect(None)
        else:
            if self._proxyModel != None:
                sel = self._proxyModel.mapToSource(sel[0])
            else:
                sel = sel[0]

            self._selectedRow = sel.row()

            self._model.doSelect(self._selectedRow)
            self.scrollTo(sel)
